package org.alltimeflashdreamer.filelister;

import java.io.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.alltimeflashdreamer.util.StringUtils;

/**
 * <p>
 *
 * Title: FileListerConfig</p> <p>
 *
 * Description: Stores the configuration settings of the current user/session.<br>
 * These settings are initialized on processing of the initial site and may be altered
 * on reloadings of the second, file-listing site.</p> <p>
 *
 * Organization: All Time Flash Dreamer</p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */

public class FileListerConfig implements Serializable
{
    /** the name of the initial site of the FileLister application */
    public final static String INITIALSITE = "definesearch.jsp";
    /** the name of the (search) results site of the FileLister application */
    public final static String RESULTSSITE = "results.jsp";

    /** the name of the controller servlet of the FileLister application */
    final static String CONTROLLERSERVLET = "FileListerControllerServlet";
    /** indicates the "new search" mode */
    final static int NEWSEARCH = 0;
    /** tells to search within the previous results */
    final static int SEARCHWITHINRESULTS = 1;
    /** indicates a new search with the old settings */
    final static int SEARCHAGAIN = 2;
    /** defines the minimum length of one list of FileListerEntry objects */
    final static int MINIMUMENTRIESINMEMORY = 100;
    final static String[] SORTCRITERIA = {"type", "name", "directory", "size", "lastmodified"};

    private final static int DEFAULTRESULTCOUNT = 25;
    private final static String[] SORTCRITERIAOUTPUT = {"File/Folder", "Name", "Location", "Size", "Last modified"};
    private final static String[] SORTORDERS = {"asc", "desc", "none"};
    private final static String DEFAULTSORTORDER = SORTCRITERIA[1] + SORTORDERS[0];
    private final static String VIEWERSITE = "viewresults.jsp";
    private final static String VIEWERSERVLET = "FileListerViewerServlet";
    private final static String ERRORSITE = "errorsite.jsp";
    private final static String DOWNLOADSITE = "download.jsp";
    private final static String DOWNLOADSERVLET = "FileListerDownloadServlet";
    private final static String OPTIONBARSITE = "optionbar.jsp";

    private boolean bIsArchiveMode = true;
    private String sStartDirectory = "";
    private FileLister fileLister = null;
    private ArrayList arraySortOrders = new ArrayList();
    private int iResultsPerSite = DEFAULTRESULTCOUNT;
    private int iStartIndex = 0;//default result row/entry to show
    private PropertyFileParameters params;
    private boolean bIsInitialArchivingProcess = false;
    private FileListerException fle = null;
    private ArrayList arraySearchParams = new ArrayList();
    private String sZipFileName;
    private long lLastTimestamp = 0L;
    private boolean bIsAdvancedSearch = false;
    private DefineSearchParameters lastDSP;
    private int iWindowID;
    private ZipBuilderThread zipBuilderThread;
    private String sSearchType;
    private boolean bDefineSearchCalledLegally = true;

    /**
     * The constructor initializes the class and constructs the URL's of each site
     * to be shown.
     *
     * @param request    the request to get the request URL from
     * @param context    the servlet context to get the full (physical) paths for
     *      each DimDom template file
     * @param params     the object containing the property file parameters
     * @param iWindowID  the id of the session window to which this instance belongs
     *      (generated by the FileListerFactory)
     */
    FileListerConfig( HttpServletRequest request, ServletContext context,
            PropertyFileParameters params, int iWindowID )
    {
        request.getContextPath();
        this.params = params;
        this.iWindowID = iWindowID;
        addDefaultSortOrder();
    }

    /**
     * This constructor may be called from other classes than the FileListerViewerServlet,
     * as used to build the initial index.
     *
     * @param params  the object containing the property file parameters
     */
    FileListerConfig( PropertyFileParameters params )
    {
        this.params = params;
        bIsInitialArchivingProcess = true;
    }

    /**
     * Tells whether the directories shall be listed.
     *
     * @return   list the directories?
     */
    public boolean isDirectoryListing()
    {
        try
        {
            return getLastSearchParam().isDirectoryListing();
        } catch ( NullPointerException npe )
        {
            return false;
        }
    }

    /**
     * Tells whether the files shall be listed.
     *
     * @return   list the files?
     */
    public boolean isFileListing()
    {
        try
        {
            return getLastSearchParam().isFileListing();
        } catch ( NullPointerException npe )
        {
            return false;
        }
    }

    /**
     * Returns wether the "zip.isallowedtozip" switch is set to true in the property
     * file or not. If true, the GUI shows the "add to zip" checkboxes and the "get
     * zip file" button, else not.
     *
     * @return   is allowed to (download) zip (files)?
     */
    public boolean isAllowedToZip()
    {
        return params.isAllowedToZip();
    }

    /**
     * Tells whether the the archive mode or the directory input mode is active.
     *
     * @return   archive or directory input mode?
     */
    public boolean isArchiveMode()
    {
        return bIsArchiveMode;
    }

    /**
     * Returns the start directory of the (last) search as a string, formatted either
     * absolutely or relatively, as defined in the properties file
     *
     * @return                      the search directory string
     * @throws FileListerException  in case the virtual root is not the beginning
     *      of the directory entries in the database, which means that the index
     *      is not up to date
     */
    public String getSearchDirectory()
        throws FileListerException
    {
        return getDirectoryOutput( sStartDirectory );
    }

    /**
     * Notifies the config class about the type of the next search. Possible types
     * are: NEWSEARCH, SEARCHAGAIN, SEARCHWITHINRESULTS.<br>
     * The first type leads to a full reset of the class' parameters, the second
     * keeps the settings, but allows to enter a new directory, the third keeps anything
     * so that the search is done within the same results as before.
     */
    void createDefineSearchParameters()
    {
        //find out the search mode
        int iSearchMode = getSearchMode();

        //store DefineSearchParameters instance before doing anything else!
        DefineSearchParameters dsp = null;
        if ( iSearchMode == NEWSEARCH )
        {
            sStartDirectory = "";
            bIsAdvancedSearch = false;
            dsp = new DefineSearchParameters( iSearchMode );
        }
        else
            dsp = new DefineSearchParameters( getLastSearchParam(), iSearchMode );

        //reset other attributes, if necessary
        FileLister fileLister = null;
        bIsInitialArchivingProcess = false;
        iStartIndex = 0;//default result row/entry to show
        fle = null;
        if ( iSearchMode == NEWSEARCH || iSearchMode == SEARCHWITHINRESULTS )
            bIsArchiveMode = true;
        if ( iSearchMode == NEWSEARCH || iSearchMode == SEARCHAGAIN )
            arraySearchParams.clear();

        //store the newly created instance (for reuse during change of simple/advanced
        //search mode)
        lastDSP = dsp;
    }

    /**
     * Returns the previously created DefineSearchParameters instance.
     *
     * @return   the current/last search configuration object
     */
    public DefineSearchParameters getDefineSearchParameters()
    {
        return lastDSP;
    }

    /**
     * Returns the index time of the search directory (which is taken as the index
     * time of all results!).
     *
     * @return   the formatted index time, as a String
     */
    public String getIndexTime()
    {
        try
        {
            Date dtIndexTime = new Date( getFileLister().getLastIndexed() );
            return DateFormat.getDateInstance( DateFormat.MEDIUM ).format( dtIndexTime ) + "<br>" +
                    DateFormat.getTimeInstance( DateFormat.MEDIUM ).format( dtIndexTime );
        } catch ( NullPointerException npe )
        {
            //may happen before the first initialization of a FileLister!
            return "";
        }
    }

    /**
     * Returns the value of the property file parameter "link.regexphelp", to be
     * shown in the "define search" site.
     *
     * @return   as defined in the property file
     */
    public String getRegExpHelpLink()
    {
        return params.getRegExpHelpLink();
    }

    /**
     * Returns whether all contents shall be listed (client entered the empty string
     * as the search string) or not.
     *
     * @return   whether to list all contents or filtered ones only
     */
    public boolean isListingAllEntries()
    {
        try
        {
            return getLastSearchParam().isListingAllEntries();
        } catch ( NullPointerException npe )
        {
            return false;
        }
    }

    /**
     * Returns either the URL of the controller servlet or the URL of the results
     * site, depending on bOfInitialSite. The returned URL is followed by "?window="
     * + getWindowID to make sure that the correct instance of FileListerConfig is
     * mapped to the link target.
     *
     * @param bOfInitialSite  show initial site (via the controller servlet)?
     * @return                the appropriate URL
     * @see                   #getWindowID
     */
    public String getRequestURL( boolean bOfInitialSite )
    {
        return appendWindowID( ( bOfInitialSite ) ? CONTROLLERSERVLET : RESULTSSITE );
    }

    /**
     * Returns the URL of the viewer servlet. The returned URL is followed by "?window="
     * + getWindowID to make sure that the correct instance of FileListerConfig is
     * mapped to the link target.
     *
     * @return   the URL of the viewer servlet
     * @see      #getWindowID
     */
    public String getViewerServletURL()
    {
        return appendWindowID( VIEWERSERVLET );
    }

    /**
     * Returns the URL of the initial site. The returned URL is followed by "?window="
     * + getWindowID to make sure that the correct instance of FileListerConfig is
     * mapped to the link target.
     *
     * @return   the URL of the initial site
     * @see      #getWindowID
     */
    public String getInitialSiteURL()
    {
        return appendWindowID( INITIALSITE );
    }

    /**
     * Returns the URL of the error site, to allow the redirect to this site. Thereturned
     * URL is followed by "?window=" + getWindowID to make sure that the correct
     * instance of FileListerConfig is mapped to the link target.
     *
     * @return   the appropriate URL
     * @see      #getWindowID
     */
    public String getErrorURL()
    {
        return appendWindowID( ERRORSITE );
    }

    /**
     * Returns the full URL of the download servlet, ready to be placed into e.g.
     * the form tag of the results site.
     *
     * @return   the fully qualified URL of the download servlet
     * @see      #getWindowID
     */
    public String getDownloadURL()
    {
        return appendWindowID( DOWNLOADSERVLET );
    }

    /**
     * Returns the full URL of the option bar JSP site, ready to be placed into e.g.
     * the lower frame tag of the results site.
     *
     * @return   the fully qualified URL of the option bar JSP site
     * @see      #getWindowID
     */
    public String getOptionBarURL()
    {
        return appendWindowID( OPTIONBARSITE );
    }

    /**
     * Returns the current results-per-site count.
     *
     * @return   the results-per-site count
     */
    public int getResultsPerSite()
    {
        return iResultsPerSite;
    }

    /**
     * Returns whether the user selected to use regular expressions for the search
     * string or not.
     *
     * @return   user's using regexp?
     */
    public boolean isUsingRegExp()
    {
        try
        {
            return getLastSearchParam().isUsingRegExp();
        } catch ( NullPointerException npe )
        {
            return false;
        }
    }

    /**
     * Returns whether the search string shall be handled in the case sensitive fashion
     * or not.
     *
     * @return   case sensitive compares?
     */
    public boolean isCaseSensitiveSearch()
    {
        try
        {
            return getLastSearchParam().isCaseSensitiveSearch();
        } catch ( NullPointerException npe )
        {
            return false;
        }
    }

    /**
     * Returns whether the search string shall be matched exactly or not.
     *
     * @return   case sensitive compares?
     */
    public boolean isExactMatch()
    {
        try
        {
            return getLastSearchParam().isExactMatch();
        } catch ( NullPointerException npe )
        {
            return false;
        }
    }

    /**
     * Returns the total amount of sites that may be shown (of course only 1, if
     * set to "show unlimited results").
     *
     * @return   the total site count (at least 1)
     */
    public int getTotalSiteCount()
    {
        if ( getFileLister() == null || getFileLister().getResultCount() == 0 )
            return 1;
        else if ( ( getFileLister().getResultCount() % getResultsPerSite() ) == 0 )
            return ( getFileLister().getResultCount() / getResultsPerSite() );
        else
            return ( ( getFileLister().getResultCount() / getResultsPerSite() ) + 1 );
    }

    /**
     * Returns the index of the currently shown site.
     *
     * @return   the site index (at least 1)
     */
    public int getCurrentSiteNumber()
    {
        return ( iStartIndex / getResultsPerSite() + 1 );
    }

    /**
     * Returns the currently set FileListerException instance or null, if none. Because
     * FileListerException extends ServletException, it implements the getRootException
     * mechanism too, to allow the user to see what happened effectively.
     *
     * @return   the currently set FileListerException or null
     */
    public FileListerException getFileListerException()
    {
        return this.fle;
    }

    /**
     * Allows to set a FileListerException from anywhere, including JSP sites.
     *
     * @param fle  which error happened (root causes are stored internally)
     */
    public void setFileListerException( FileListerException fle )
    {
        this.fle = fle;
    }

    /**
     * Returns a 100th of the max. FileListerEntry objects parameter of the config
     * file, but at least 100. This value is used as the buffer size by the FileLister
     * class and defines how many results are loaded at a time
     *
     * @return   the (current) FileListerEntry buffer's size
     */
    public int getBufferSize()
    {
        return ( ( ( getMaxFileListerObjects() / MINIMUMENTRIESINMEMORY ) < MINIMUMENTRIESINMEMORY )
                ? MINIMUMENTRIESINMEMORY
                : getMaxFileListerObjects() / MINIMUMENTRIESINMEMORY );
    }

    /**
     * Returns whether the advanced mode (which includes size and date search input
     * in the GUI) is active or not.
     *
     * @return   advanced search?
     */
    public boolean isAdvancedSearch()
    {
        return bIsAdvancedSearch;
    }

    /**
     * Parses the URL input value; if "simple" or "advanced", the search mode parameter
     * is updated to the new mode setting.
     *
     * @param sURLInput  handled only if equals "simple" or "advanced"
     */
    public void setSearchMode( String sURLInput )
    {
        if ( sURLInput.equals( "simple" ) )
            setAdvancedSearch( false );
        else if ( sURLInput.equals( "advanced" ) )
            setAdvancedSearch( true );
    }

    /**
     * Returns the window id (or the number of this class within the user's session)
     * that was set during initialization.
     *
     * @return   the window id representing this class
     */
    public int getWindowID()
    {
        return iWindowID;
    }

    /**
     * Builds the HTML to be put into a select box for the sort orders, specified
     * within the sort orders by the given index (from 1 up to the total amount of
     * defined sort orders). If the given index <i>is higher than 1</i> , the first
     * option in the list is empty, having the URL value "none". The URL value of
     * all the other entries is the same as the visible output.<br>
     * The method is settled within FileListerConfig (and not within SearchParameters
     * like the other "Output" methods) because of the special situation with the
     * sort orders: There kept in / controlled by this class (because they're overall
     * settings), unlike all the other search parameters that form the SQL where
     * conditions.
     *
     * @param iSortOrderNumber  the number of the sort criterium within the list
     *      of sort criteria, starting with 1
     * @return                  the full HTML string to fill the iSortOrderNumber-th
     *      select box with an option list
     */
    public String getSortOrderOutput( int iSortOrderNumber )
    {
        StringBuffer sb = new StringBuffer();
        if ( iSortOrderNumber > 1 )
            sb.append( "<option value=\"none\"></option>" );
        for ( int i = 0; i < SORTCRITERIA.length; i++ )
        {
            sb.append( "<option value=\"" ).append( SORTCRITERIA[i] ).append( "\"" );
            if ( arraySortOrders.size() >= iSortOrderNumber )
            {
                if ( ( ( String )arraySortOrders.get( iSortOrderNumber - 1 ) ).startsWith( SORTCRITERIA[i] ) )
                    sb.append( " selected=\"on\"" );
            }
            sb.append( ">" ).append( SORTCRITERIAOUTPUT[i] ).append( "</option>" );
        }
        return sb.toString();
    }

    /**
     * Asks the ZipBuilderThread for its generation/progress percentage, formats
     * it to one fraction digit and appends a % sign to it. The fully formatted string
     * is returned.
     *
     * @return   e.g. "17.9%"
     */
    public String getZipProgressPercentage()
    {
        if ( getZipBuilderThread() == null )
            return "";

        NumberFormat nf = NumberFormat.getInstance();
        nf.setMinimumFractionDigits( 1 );
        nf.setMaximumFractionDigits( 1 );
        return nf.format( 100.0 * getZipBuilderThread().getProgressPercentage() ) + "%";
    }

    /**
     * Returns the list of warnings generated while creating the zip file. Such a
     * warning might be the information that a selected entry is missing or that
     * a file is contained in the zip that hasn't been in the selection before, because
     * the index is somewhat out of sync.
     *
     * @return   a list of warnings, one per line (if any, else "")
     */
    public String getZipWarnings()
    {
        if ( getZipBuilderThread() == null )
            return "";

        ArrayList arrayWarnings = getZipBuilderThread().getWarnings();
        if ( arrayWarnings.isEmpty() )
            return "";

        StringBuffer sbWarnings = new StringBuffer( "Warnings:\n" );
        for ( int i = 0; i < arrayWarnings.size(); i++ )
        {
            sbWarnings.append( ( String )arrayWarnings.get( i ) );
            if ( i < ( arrayWarnings.size() - 1 ) )
                sbWarnings.append( "\n" );
        }
        return sbWarnings.toString();
    }

    /**
     * Returns the descriptor for the current archive size of the zip file the user
     * is waiting for.
     *
     * @return   something like "...archive size: 5.23 MB (13% packed)"
     */
    public String getZipArchiveSize()
    {
        if ( getZipBuilderThread() == null )
            return "";

        NumberFormat nf = NumberFormat.getInstance();
        nf.setMinimumFractionDigits( 0 );
        nf.setMaximumFractionDigits( 1 );
        return new StringBuffer( "...archive size: " ).append( StringUtils.formatSize( getZipBuilderThread().getCurrentArchiveSize() ) )
                .append( " (" ).append( nf.format( 100.0 * getZipBuilderThread().getOverallPackingRatio() ) ).append( "% packed)" )
                .toString();
    }

    /**
     * Returns the descriptor of the progress of files packed into the current zip.
     *
     * @return   something like "29 of 314 files done (215.2 KB of 2.234 MB)"
     */
    public String getZipFilesDone()
    {
        if ( getZipBuilderThread() == null )
            return "";

        return getZipBuilderThread().getFilesDone() + " files done (" + getZipBuilderThread().getSizeDone() + ")";
    }

    /**
     * Returns the currently-added-file descriptor.
     *
     * @return   e.g. "Currently adding: /temp/xyz.gif"
     */
    public String getZipCurrentlyAdding()
    {
        if ( getZipBuilderThread() == null )
            return "";

        return "currently adding: " + getZipBuilderThread().getCurrentlyAdding();
    }

    /**
     * Returns the descriptor for the time already elapsed since the start of the
     * zip building.
     *
     * @return   e.g. "time elapsed: 1 min 27s (less than 1 minute remaining)"
     */
    public String getZipTimeElapsed()
    {
        if ( getZipBuilderThread() == null )
            return "";

        return getZipBuilderThread().getTimeElapsed();
    }

    /**
     * Returns the content of the meta-refresh tag (used by download.jsp only).
     *
     * @return   the modified meta-refresh tag, having either 0 or 2 seconds delay,
     *      depending on whether the zip file is built already or not
     */
    public String getMetaRefreshContent()
    {
        return ( ( isReadyToStream() ) ? "0" : "2" ) + "; URL=" + getDownloadURL();
    }

    /**
     * Returns the current width of the progress bar describing the build status
     * of the zip file (used by download.jsp only), formatted like "xy%".
     *
     * @return   something like "xy%", where xy is from 0 up to 100
     */
    public String getProgressBarWidth()
    {
        return ( isReadyToStream() ) ? "100%" : getZipProgressPercentage();
    }

    /**
     * Reads the URL parameters and stores those values within an instance of SearchParameters.
     * This instance itself is stored in the internal list of SearchParameters instances.
     *
     * @param request               the request to get the search parameters from
     * @throws FileListerException  wraps/describes the possibly upcoming input errors
     */
    void createSearchParameters( HttpServletRequest request )
        throws FileListerException
    {
        //find out whether the definesearch.jsp was called "regularly" or via the
        //back-button of the browser; in the latter case, the createDefineSearchParameters
        //method must be called before continuing, ignoring the returned object
        try
        {
            long lCurrentTimestamp = new Long( request.getParameter( "timestamp" ) ).longValue();
            if ( lCurrentTimestamp == lLastTimestamp )
            {
                //same timestamp as during the last call -> back-button was pressed
                setSearchType( request.getParameter( "search" ) );
                createDefineSearchParameters();
            }
            else
                lLastTimestamp = lCurrentTimestamp;
        } catch ( Exception ex )
        {
            lLastTimestamp = System.currentTimeMillis();
        }

        arraySearchParams.add( new SearchParameters( request ) );

        //get the selected search mode (index or directory search), check against abuse and
        //verify the directory input, in case the latter mode was selected
        String sSearchMode = request.getParameter( "searchmode" );
        if ( StringUtils.isEmpty( sSearchMode ) )
            throw new FileListerException( 2 );
        else if ( sSearchMode.equals( "directory" ) )
            bIsArchiveMode = false;
        else if ( sSearchMode.equals( "archive" ) )
            bIsArchiveMode = true;
        else
            throw new FileListerException( 3 );

        //start directory: sets the sStartDirectory pointer
        buildStartDirectory( request.getParameter( "directory" ) );
    }

    /**
     * Returns the (validated, but nevertheless abstract) directory defined by the
     * client/user, appended to the root directory value from the property file,
     * if defined.
     *
     * @return   the "fully qualified" search directory
     */
    File getStartDirectory()
    {
        return new File( sStartDirectory );
    }

    /**
     * Tells whether absolute or relative paths shall be shown in the output.
     *
     * @return   show absolute or relative paths?
     */
    boolean isShowingAbsolutePaths()
    {
        return params.isShowingAbsolutePaths();
    }

    /**
     * Returns the list of SearchParameters instances entered by the client.
     *
     * @return   the user's search string(s) or the empty String, as a List
     */
    ArrayList getSearchParameters()
    {
        return arraySearchParams;
    }

    /**
     * Sets the FileLister instance which contains the (filtered) list of files and/or
     * directories that match the user's search criteria.<br>
     * If the given parameter is null, this leads to an unsetting of the list.
     *
     * @param fileLister  the previously created FileLister instance or null
     */
    void setFileLister( FileLister fileLister )
    {
        this.fileLister = fileLister;
    }

    /**
     * Returns the FileLister, if set previously, or null.
     *
     * @return   the stored FileLister or null
     */
    public FileLister getFileLister()
    {
        return fileLister;
    }

    /**
     * Returns the fully qualified file name of the JSP download site which is filled
     * dynamically to show a virtual progress bar and other interesting remarks during
     * zip building.
     *
     * @return   the formatted download site URL
     */
    String getDownloadSite()
    {
        return appendWindowID( DOWNLOADSITE );
    }

    /**
     * Returns the fully qualified file name of the JSP results site which is filled
     * dynamically with the found entries (currently visible subset).
     *
     * @return   the formatted results site URL
     */
    String getViewerSite()
    {
        return appendWindowID( VIEWERSITE );
    }

    /**
     * Verifies the URL input and sets the new sort mode.<br>
     * Allowed values are: name, directory, size, lastmodified, each followed by
     * either asc or desc. The sorting work is minimized as much as possible...
     *
     * @param sURLInput             the URL input for the new sort mode
     * @throws FileListerException  thrown on bad or missing input
     */
    void sortByColumn( String sURLInput )
        throws FileListerException
    {
        if ( StringUtils.isEmpty( sURLInput ) )
            throw new FileListerException( 11 );
        else if ( sURLInput.equals(arraySortOrders.get( 0 )) )
            return;
        else if ( isValidSortCriterium( sURLInput ) && isValidSortOrder( sURLInput ) )
        {
            //each new sorting is done by forcing the FileLister to requery the
            //database (AFTER the list of sort criteria has been updated!)
            reloadFileLister( sURLInput );
        }
        else
            throw new FileListerException( 11 );
    }

    /**
     * Returns the list currently active sort criteria. Each entry is a String that
     * starts with either name or location or size or lastmodified and ends with
     * asc or desc.
     *
     * @return   the current list of sort criteria
     */
    ArrayList getSortOrders()
    {
        return arraySortOrders;
    }

    /**
     * Returns the URL parameter value of the "column" parameter shown in the link
     * of each sortable table column header. The returned value consists of the column
     * name itself, followed by either "asc", "desc" or "none". "Next" means, that
     * the current sort order is asked and the follower is returned, i.e. if not
     * a sort criterium (= "none" or not existent), "asc" is returned, then "desc"
     * and finally "none".
     *
     * @param sColumnName  which column's sort order shall be returned
     * @return             the URL parameter value, as described above
     */
    public String getNextSortOrder( String sColumnName )
    {
        String sNextEntry = null;
        for ( int i = 0; i < arraySortOrders.size(); i++ )
        {
            sNextEntry = ( String )arraySortOrders.get( i );
            if ( sNextEntry.startsWith( sColumnName ) )
            {
                if ( sNextEntry.endsWith( SORTORDERS[0] ) )
                    return sColumnName + SORTORDERS[1];
                else if ( sNextEntry.endsWith( SORTORDERS[1] ) )
                    return sColumnName + SORTORDERS[2];
                else if ( sNextEntry.endsWith( SORTORDERS[2] ) )
                    return sColumnName + SORTORDERS[0];
            }
        }

        //if the method comes to here, the current column hasn't been
        //a sort criterium yet -> return "asc"
        return sColumnName + SORTORDERS[0];
    }

    /**
     * Scans the list of currently stored sort criteria for an entry starting with
     * sColumnName. If such an entry exists, true is returned, else false.
     *
     * @param sColumnName  the column name to check
     * @return             whether this column name is stored to be a sort criterium
     *      or not
     */
    public boolean isSortCriterium( String sColumnName )
    {
        for ( int i = 0; i < arraySortOrders.size(); i++ )
        {
            if ( ( ( String )arraySortOrders.get( i ) ).startsWith( sColumnName ) )
                return true;
        }
        return false;
    }

    /**
     * Scans the list of currently stored sort criteria for an entry starting with
     * sColumnName. If such an entry exists, its sort direction is returned ("asc",
     * "desc" or "none"), if not, "none" is returned.
     *
     * @param sColumnName  the column name to check
     * @return             the sort direction of this column
     */
    public String getSortDirection( String sColumnName )
    {
        String sNextEntry = null;
        for ( int i = 0; i < arraySortOrders.size(); i++ )
        {
            sNextEntry = ( String )arraySortOrders.get( i );
            if ( sNextEntry.startsWith( sColumnName ) )
                return sNextEntry.substring( sColumnName.length() );
        }
        return SORTORDERS[2];
    }

    /**
     * Returns the inverse sort direction of the given column, i.e. asc, if it's
     * currently set to desc and vice versa. If no sorting is set to this column,
     * none is returned.
     *
     * @param sColumnName  the column name to check
     * @return             the inverse sort direction of this column or "none"
     */
    public String getInverseSortOrder( String sColumnName )
    {
        String sNextEntry = null;
        for ( int i = 0; i < arraySortOrders.size(); i++ )
        {
            sNextEntry = ( String )arraySortOrders.get( i );
            if ( sNextEntry.startsWith( sColumnName ) )
            {
                if ( sNextEntry.endsWith( SORTORDERS[0] ) )
                    return sColumnName + SORTORDERS[1];
                else if ( sNextEntry.endsWith( SORTORDERS[1] ) )
                    return sColumnName + SORTORDERS[0];
            }
        }

        //return "none" as default
        return sColumnName + SORTORDERS[2];
    }

    /**
     * Scans the list of currently stored sort criteria for an entry starting with
     * sColumnName. If such an entry exists, its index within the list of criteria,
     * i.e. the sort priority, is returned. is returned.
     *
     * @param sColumnName  the column name to check
     * @return             the sort priority of the given column or 0, if not a sort
     *      criterium
     */
    public int getSortPriority( String sColumnName )
    {
        for ( int i = 0; i < arraySortOrders.size(); i++ )
        {
            if ( ( ( String )arraySortOrders.get( i ) ).startsWith( sColumnName ) )
                return ( i + 1 );
        }
        return 0;
    }

    /**
     * Returns a string describing the current configuration settings.
     *
     * @return                      ready to be shown on the viewer site
     * @throws FileListerException  in case the virtual root is not the beginning
     *      of the directory entries in the database, which means that the index
     *      is not up to date
     */
    public String getConfigDescriptor()
        throws FileListerException
    {
        //list search string(s)
        StringBuffer sb = new StringBuffer( "Search String: " ).append( getSearchStringDescriptor() );

        //list search directory
        sb.append( "<br>Search in: " ).append( getSearchDirectory() );

        //list size and date descriptors, if necessary
        SearchParameters params = getLastSearchParam();
        if ( params.getMinimumSize() >= 0L || params.getMaximumSize() >= 0L )
            sb.append( "<br>Size: " ).append( getSizeDescriptor( params ) );
        if ( params.getStartingDate() != null || params.getEndingDate() != null )
            sb.append( "<br>Date: " ).append( getDateDescriptor( params ) );

        //list result count, depending on whether the user wants to see
        //unlimited results or not
        sb.append( "<br>Results: " );
        int iNRecords = getFileLister().getResultCount();
        if ( iNRecords == 0 )
            sb.append( "(none)" );
        else
        {
            if ( iNRecords > 1 )
                sb.append( getStartIndex() + 1 ).append( " - " );
            sb.append( ( iNRecords > getStartIndex() + getResultsPerSite() ) ? getStartIndex() + getResultsPerSite() : iNRecords )
                    .append( " out of " ).append( iNRecords );
        }

        return sb.toString();
    }

    /**
     * Shows the directory string that is <i>visible in the web output</i> . This
     * is not the absolute path only if directory.showabsolutepaths is set to false
     * AND a virtual root is defined. In this case, the path is relative to this
     * virtual root.
     *
     * @param sDirectory            the directory to transform
     * @return                      the virtual directory or sDirectory itself
     * @throws FileListerException  in case the virtual root is not the beginning
     *      of the directory entries in the database, which means that the index
     *      is not up to date
     */
    public String getDirectoryOutput( String sDirectory )
        throws FileListerException
    {
        //need to cut the virtual root from sDirectory?
        if ( hasVirtualRoot() && !isShowingAbsolutePaths() )
        {
            String sTempDir = sDirectory;
            String sTempRoot = getVirtualRoot();
            if ( FileLister.ISWINDOWS )
            {
                sTempDir = StringUtils.replaceBackslashes( sDirectory );
                sTempRoot = StringUtils.replaceBackslashes( getVirtualRoot() );
            }

            //make sure that it's possible that sTempDir starts with sTempRoot (at least a Linux problem...)
            sTempDir = StringUtils.appendFileSeparator( sTempDir, true );

            if ( sTempDir.equals( sTempRoot ) )
                return "/";
            else if ( sTempDir.startsWith( sTempRoot ) )
            {
                String sReturnValue = StringUtils.replaceBackslashes( sTempDir.substring( sTempRoot.length() ) );
                return ( ( sReturnValue.startsWith( "/" ) ) ? sReturnValue : "/" + sReturnValue );
            }
            else if ( sTempRoot.endsWith( sTempDir ) )
            {
                //can come to here only on very special/bad user behaviour and occurs in
                //the "previous error reported" situation only; because in this case a
                //virtual root is defined anyway, we define that the virtual root shall
                //be shown then...
                return "/";
            }
            else
            {
                //paths don't match, although they definitely should: verify case "/" vs. "E:/"
                throw new FileListerException( 32, new FileNotFoundException( "directory mismatch: search directory = " +
                        sTempDir + ", virtual root directory = " + sTempRoot ) );
            }
        }

        return sDirectory;
    }

    /**
     * Redirects the client to a specific site. If the URL input value is not a number,
     * an exception is thrown. If it's &lt; 1, it's set to 1, if it's &gt; than the
     * last site, it's set to the last possible site number.
     *
     * @param sURLInput             what the user entered in the jump-to-site text
     *      field
     * @throws FileListerException  if the input is not a number
     */
    void jumpToSite( String sURLInput )
        throws FileListerException
    {
        int iTargetSiteNumber = 1;
        try
        {
            iTargetSiteNumber = new Integer( sURLInput ).intValue();
        } catch ( Exception ex )
        {
            throw new FileListerException( 12 );
        }

        //don't navigate if not necessary: unlimited or only a single site
        if ( getResultsPerSite() >= getFileLister().getResultCount() )
            return;

        //navigation: use as much of the navigate method as possible (first and last site)
        if ( iTargetSiteNumber <= 1 )
            navigate( "first" );
        else if ( iTargetSiteNumber > ( getFileLister().getResultCount() / getResultsPerSite() ) )
            navigate( "last" );
        else
        {
            //transform target site to counter starting with 0 and multiply it with the
            //currently set amount of results per site
            iStartIndex = ( iTargetSiteNumber - 1 ) * getResultsPerSite();

            //notify the file lister about the new start index; this allows
            //the file lister to prepare its FileListerEntry buffer, if necessary
            getFileLister().prepareEntries( iStartIndex );
        }
    }

    /**
     * Verifies the URL input and sets the new results-per-site count.<br>
     * Allowed values are: 10, 25, 50 and the value returned by the getBufferSize
     * method.
     *
     * @param sURLInput             the URL input for the new results-per-site count
     * @throws FileListerException  thrown on bad input
     */
    void setResultsPerSite( String sURLInput )
        throws FileListerException
    {
        if ( StringUtils.isEmpty( sURLInput ) )
            throw new FileListerException( 11 );

        if ( sURLInput.equals( "10" ) || sURLInput.equals( "25" ) ||
                sURLInput.equals( "50" ) || sURLInput.equals( "" + getBufferSize() ) )
        {
            iResultsPerSite = new Integer( sURLInput ).intValue();
            if ( ( getStartIndex() % getResultsPerSite() ) != 0 )
                iStartIndex -= getStartIndex() % getResultsPerSite();//avoid invalid start indexes of each site
        }
        else
            throw new FileListerException( 11 );
    }

    /**
     * Returns the index from which on the results shall be shown (default is 0,
     * i.e. the first entry, but this value may change during navigation within the
     * results).
     *
     * @return   the start/base index to show the appropriate results
     */
    public int getStartIndex()
    {
        return iStartIndex;
    }

    /**
     * Returns the link prefix parameter from the config file (may be empty).
     *
     * @return   the link prefix or ""
     */
    public String getLinkPrefix()
    {
        return params.getLinkPrefix();
    }

    /**
     * Navigates according to the URL input parameter: to the first, to the previous,
     * to the next or to the last position.
     *
     * @param sURLParameterName     how to navigate
     * @throws FileListerException  if the URL input is not a valid parameter
     */
    void navigate( String sURLParameterName )
        throws FileListerException
    {
        if ( sURLParameterName.startsWith( "first" ) )
            iStartIndex = 0;
        else if ( sURLParameterName.startsWith( "previous" ) )
        {
            iStartIndex -= getResultsPerSite();
            if ( iStartIndex < 0 )
                iStartIndex = 0;
        }
        else if ( sURLParameterName.startsWith( "next" ) )
        {
            iStartIndex += getResultsPerSite();
            if ( iStartIndex >= getFileLister().getResultCount() )
                iStartIndex = getStartIndexOfLastPage();
        }
        else if ( sURLParameterName.startsWith( "last" ) )
            iStartIndex = getStartIndexOfLastPage();
        else
            throw new FileListerException( 11 );

        //notify the file lister about the new start index; this allows
        //the file lister to prepare its FileListerEntry buffer, if necessary
        getFileLister().prepareEntries( iStartIndex );
    }

    /**
     * Returns value of the ini-file parameter "index.aging" which defines how old
     * database values are allowed to be before the directory search is started again
     * (in seconds).
     *
     * @return   seconds until next directory search is executed
     */
    int getIndexAging()
    {
        return params.getIndexAging( true );
    }

    /**
     * Returns the value of the ini-file parameter "directory.root", if defined.
     * If it's defined, it indicates the directory root which is allowed to be searched/listed
     * only, to restrict the access to the file system to a single directory.
     *
     * @return   the root of the searchable directory or null, if not defined in
     *      the property file
     */
    String getVirtualRoot()
    {
        return params.getVirtualRootDirectory( true );
    }

    /**
     * Returns whether the property file contains a virtual root or not.
     *
     * @return   virtual root defined?
     */
    boolean hasVirtualRoot()
    {
        return !getVirtualRoot().equals( "" );
    }

    /**
     * Returns the list of directories, inside which searching is prohibited.
     *
     * @return   list of directories, where searching is prohibited.
     */
    LinkedList getNoGoAreas()
    {
        return params.getNoGoAreas( true );
    }

    /**
     * Returns whether this config object was initialized to run the initial archiving
     * process or not.
     *
     * @return   run initial archiving (i.e. indexing) process?
     */
    boolean isInitialArchivingProcess()
    {
        return bIsInitialArchivingProcess;
    }

    /**
     * Returns the max. number of file lister objects to be stored in the memory,
     * as defined in the property file.
     *
     * @return   max. number of FileListerEntry objects
     */
    int getMaxFileListerObjects()
    {
        return params.getMaxFileListerObjects( true );
    }

    /**
     * Returns the value of the "jdbc.tablename" property of the config file.
     *
     * @return   the name of the table
     */
    String getJDBCTableName()
    {
        return params.getJDBCTableName();
    }

    /**
     * Returns the query builder instance defined in the property file.
     *
     * @return   the database-related, SQLQueryBuilder-implementing query builder
     *      class which is able to form the SQL strings in the database-specific
     *      format
     */
    SQLQueryBuilder getQueryBuilder()
    {
        return params.getSQLQueryBuilder();
    }

    /**
     * Returns the directory path, ending with a "/", to write the created zip files
     * in.
     *
     * @return   the zip directory
     */
    String getZipCreationDirectory()
    {
        return params.getZipCreationDirectory( true );
    }

    /**
     * Returns the package strength of the download zips. This strength is commonly
     * named "level". Allowed values are 0 - 9, default is 5, in case an illegal
     * value was defined in the property file.
     *
     * @return   0 - 9, indicating the zip level / package strength
     */
    int getZipLevel()
    {
        return params.getZipPackageStrength( true );
    }

    /**
     * Steps through the currently visible FileListerEntries and marks them as selected-for-zip,
     * if their index is within iSelectedIndexes. If not, they're unmarked. This
     * stores the selection state even if the user navigated to another view/site,
     * at most until the FileLister is reinitialized or some very old entries are
     * removed from FileLister, out of memory reasons...
     *
     * @param iSelectedIndexes  the list of selected indexes or null, indicating
     *      that none was selected
     */
    void setSelectedIndexesForZip( int[] iSelectedIndexes )
    {
        FileListerEntry[] visibleEntries = fileLister.getVisibleFileListerEntries();
        for ( int i = 0; i < visibleEntries.length; i++ )
        {
            if ( iSelectedIndexes == null )
                visibleEntries[i].setSelectedForZip( false );
            else
            {
                boolean bFound = false;
                for ( int j = 0; j < iSelectedIndexes.length; j++ )
                {
                    if ( iSelectedIndexes[j] == ( getStartIndex() + i ) )
                    {
                        visibleEntries[i].setSelectedForZip( true );
                        bFound = true;
                        break;
                    }
                }
                if ( !bFound )
                    visibleEntries[i].setSelectedForZip( false );
            }
        }
    }

    /**
     * Sets the search mode (advanced or simple).
     *
     * @param bIsAdvancedSearch  if false, it's the simple, else the advanced mode
     */
    void setAdvancedSearch( boolean bIsAdvancedSearch )
    {
        this.bIsAdvancedSearch = bIsAdvancedSearch;
    }

    /**
     * Sets the given list of sort orders defined by the client to the internal list
     * of sort orders. Because the list contains only sort criteria, but no sort
     * orders, each criterium in the list is treated as "asc".
     *
     * @param sSortCriteria  the list of sort criteria from the URL input
     */
    void setSortCriteria( String[] sSortCriteria )
    {
        arraySortOrders.clear();
        for ( int i = 0; i < sSortCriteria.length; i++ )
        {
            if ( isValidSortCriterium( sSortCriteria[i] ) )
                arraySortOrders.add( sSortCriteria[i] + SORTORDERS[0] );
        }
        if ( arraySortOrders.isEmpty() )
            addDefaultSortOrder();
    }

    /**
     * Sets an instance of ZipBuilderThread indicating that the owner session of
     * this class is building a zip file. If the building of the zip file is finished,
     * this method <i>must be called again with a null pointer</i> to notify this
     * class about the finished process.
     *
     * @param zipBuilderThread  a new thread or null, if the old one's finished
     */
    void setZipBuilderThread( ZipBuilderThread zipBuilderThread )
    {
        this.zipBuilderThread = zipBuilderThread;
    }

    /**
     * Returns the current ZipBuilderThread pointer, which is null if no such thread
     * is running.
     *
     * @return   the currently running zipping thread or null
     */
    ZipBuilderThread getZipBuilderThread()
    {
        return zipBuilderThread;
    }

    /**
     * Returns whether getZipBuilderThread is existing and still working on a zip
     * file or not.
     *
     * @return   true, if existing and still building a zip file, false, if not set
     *      or finished building the zip
     */
    boolean isZipBuilding()
    {
        return ( getZipBuilderThread() != null && getZipBuilderThread().isAlive() );
    }

    /**
     * Stops the ZipBuilderThread, if one is set, including the deletion of its zip
     * file (after creation). This is done by simply marking the ZipBuilderThread
     * in a way that it cleans up itself after finishing its run method.
     */
    public void resetZipBuilderThread()
    {
        if ( getZipBuilderThread() != null )
        {
            if ( getZipBuilderThread().isAlive() )
                getZipBuilderThread().markForCleanUp();
            else
            {
                try
                {
                    getZipBuilderThread().finalize();
                } catch ( Throwable t )
                {
                }
            }
            setZipBuilderThread( null );
        }
    }

    /**
     * Returns whether there is currently a ZipBuilderThread in the pipeline and
     * this thread is ready to stream the finished zip file. If so, the method returns
     * true, else false.
     *
     * @return   is there a zip file ready to be streamed?
     */
    public boolean isReadyToStream()
    {
        try
        {
            return getZipBuilderThread().isReadyToStream();
        } catch ( NullPointerException npe )
        {
            return false;
        }
    }

    /**
     * Returns the visible version string of the FileLister application.
     *
     * @return   a String describing the app version
     */
    public String getVersion()
    {
        return "FileLister v" + FileLister.VERSIONNUMBER;
    }

    /**
     * Returns the last stored search mode (may be null).
     *
     * @return   the currently configured search mode URL parameter
     */
    public String getSearchType()
    {
        return sSearchType;
    }

    /**
     * Sets the current search type (may be null).
     *
     * @param sURLInput  the search type, read from the URL parameter "search"
     */
    void setSearchType( String sURLInput )
    {
        sSearchType = sURLInput;
    }

    /**
     * This method provides a "security" plugin: If the user didn't click on the
     * option bar link but on the back button of the browser, the former search configuration
     * must be removed (in case of "search within results"). Therefore, the boolean
     * indicates whether the call was "legal"; on each call of the FileListerControllerServlet,
     * this method is called using "true" while on each call of the results.jsp,
     * the back-button check is done and the switch is set back to "false".
     *
     * @see   #doBackButtonInBrowserCheck
     */
    void setInitialSiteWasCalledLegally()
    {
        this.bDefineSearchCalledLegally = true;
    }

    /**
     * Provides the check whether the user called the initial site "legally" or via
     * the back button in the browser only. To find this out, the following checks
     * are done: Is the call from the initial site? Is the search type currently
     * set to "search within results"? Is the "security" switch value "false"? If
     * all those checks return true, the last search configuration object is removed
     * from the internal list of configuration objects because in this case, the
     * user clicked the back browser of the button in a moment where he definitely
     * shouldn't do that.
     *
     * @param request  where to read the URL parameters for the above described checks
     *      from
     */
    public void doBackButtonInBrowserCheck( HttpServletRequest request )
    {
        if ( !StringUtils.isEmpty( request.getParameter( "definesearch" ) ) &&
                !StringUtils.isEmpty( getSearchType() ) && getSearchType().equals( "withinresults" ) &&
                !bDefineSearchCalledLegally )
        {
            removeLastSearchParametersFromList();
        }
        bDefineSearchCalledLegally = false;
    }

    /**
     * Removes the last instance of SearchParameters from the internal list of SearchParameters.
     * This method is used to correct the behaviour of FileLister under certain special
     * circumstances.
     */
    void removeLastSearchParametersFromList()
    {
        try
        {
            //remove the former config and ignore some possible array exceptions
            arraySearchParams.remove( arraySearchParams.size() - 1 );
        } catch ( Exception ex )
        {
        }
    }

    /**
     * Reacts to a click on the simple/advanced search button. This leads into the
     * creation of a "temporary" DefineSearchParameters instance whose underlying
     * SearchParameters isn't stored in the internal list of SearchParameters instances
     * (happens only on processing of a real search).
     *
     * @param sNewMode              the new mode of the define-search site
     * @param request               where to read the URL parameters from
     * @throws FileListerException  if the URL parameters are somehow invalid
     */
    void processModeChange( String sNewMode, HttpServletRequest request )
        throws FileListerException
    {
        //parse the URL parameters into a new SearchParameters instance
        SearchParameters params = new SearchParameters( request );

        //construct the special version of DefineSearchParameters
        //which copies ALL the pointers! This is indicated by passing
        //SEARCHAGAIN as the search mode (little hack...)
        lastDSP = new DefineSearchParameters( params, SEARCHAGAIN );

        //start directory: sets the sStartDirectory pointer
        buildStartDirectory( request.getParameter( "directory" ) );

        //register the change in the search mode (simple/advanced)
        setSearchMode( sNewMode );
    }

//---------------------------- private methods ---------------------------------//
    /**
     * Calculates the start index of the last site showing results. This value depends
     * on whether unlimited results shall be shown per site and on the fact that
     * the last site must not be empty, in case the result count is some multiple
     * of the getResultsPerSite() value.
     *
     * @return   the start index of the last page showing results
     */
    private int getStartIndexOfLastPage()
    {
        if ( getFileLister().getResultCount() % getResultsPerSite() == 0 )
            return getFileLister().getResultCount() - getResultsPerSite();

        return getFileLister().getResultCount() - ( getFileLister().getResultCount() % getResultsPerSite() );
    }

    /**
     * Validates the user's directory input, combined with the root directory, if
     * defined and/or the user entered a relative path. The search directory must
     * exist and be a child of the root directory (if set).
     *
     * @param sURLInput             the user's text field input
     * @throws FileListerException  if something is wrong/inacceptable
     */
    private void buildStartDirectory( String sURLInput )
        throws FileListerException
    {
        if ( StringUtils.isEmpty( sURLInput ) )
            sURLInput = "/";

        File startDirectory = null;
        if ( hasVirtualRoot() )
        {
            //check whether the directory entered by the user exists
            startDirectory = new File( getVirtualRoot() + sURLInput );
            if ( !startDirectory.exists() || !startDirectory.isDirectory() )
            {
                startDirectory = new File( sURLInput );
                if ( !startDirectory.exists() || !startDirectory.isDirectory() )
                {
                    //clear previous results, if any, to disable the navigation
                    if ( getFileLister() != null )
                        getFileLister().clear();
                    throw new FileListerException( 16 );
                }
            }

            //check whether user entered a path below the root directory; make
            //sure that the compared directory ends with a file separator too
            resolveStartDirectory( startDirectory );
            if ( !( StringUtils.appendFileSeparator( sStartDirectory, true ) ).startsWith( getVirtualRoot() ) )
                throw new FileListerException( 17 );
        }
        else
        {
            //check whether the directory entered by the user exists
            startDirectory = new File( sURLInput );
            if ( !startDirectory.exists() || !startDirectory.isDirectory() )
            {
                //clear previous results, if any, to disable the navigation
                if ( getFileLister() != null )
                    getFileLister().clear();
                throw new FileListerException( 16 );
            }
            resolveStartDirectory( startDirectory );
        }

        //DEBUG: System.out.println( "buildStartDirectory: sURLInput was: " + sURLInput + ", Start directory is: " + startDirectory.getPath() );
    }

    /**
     * Resolves the given directory's path and sets this absolute value to sStartDirectory.
     *
     * @param dir                   the new (resolved) start directory
     * @throws FileListerException  if this directory is not valid
     */
    private void resolveStartDirectory( File dir )
        throws FileListerException
    {
        try
        {
            sStartDirectory = FileLister.getRealPath( dir );
        } catch ( Exception ex )
        {
            //clear previous results, if any, to disable the navigation
            if ( getFileLister() != null )
                getFileLister().clear();
            throw new FileListerException( 16, ex );
        }
    }

    /**
     * Returns whether the given String starts with one of the allowed SORTCRITERIA
     * values or not (start only because it's ending with one of the SORTORDER values).
     *
     * @param sTestString  test this one!
     * @return             valid?
     */
    private boolean isValidSortCriterium( String sTestString )
    {
        for ( int i = 0; i < SORTCRITERIA.length; i++ )
        {
            if ( sTestString.startsWith( SORTCRITERIA[i] ) )
                return true;
        }
        return false;
    }

    /**
     * Returns whether the given String ends with one of the allowed SORTORDERS values
     * or not.
     *
     * @param sTestString  test this one!
     * @return             valid?
     */
    private boolean isValidSortOrder( String sTestString )
    {
        for ( int i = 0; i < SORTORDERS.length; i++ )
        {
            if ( sTestString.endsWith( SORTORDERS[i] ) )
                return true;
        }
        return false;
    }

    /**
     * Updates the (complex) list of sort criteria and tells the FileLister to requery
     * the database and rebuild the list of FileListerEntry objects.
     *
     * @param sURLInput             descriptor of the sorting action
     * @throws FileListerException  thrown by FileLister's reloading of the database
     */
    private void reloadFileLister( String sURLInput )
        throws FileListerException
    {
        //already selected criterium?
        if ( arraySortOrders.contains( sURLInput ) )
            return;

        String[] sCriteria = ( String[] )arraySortOrders.toArray( new String[]{} );

        //sURLInput ends with none? -> remove it from the list
        if ( sURLInput.endsWith( SORTORDERS[2] ) )
        {
            for ( int i = 0; i < sCriteria.length; i++ )
            {
                if ( sCriteria[i].startsWith( sURLInput.substring( 0, 4 ) ) )
                {//minimum size of a column name

                    arraySortOrders.remove( i );
                    if ( arraySortOrders.size() == 0 )
                        addDefaultSortOrder();
                    break;
                }
            }
        }
        //else find the one starting with the same criterium; if existent, replace
        //it, else add it add the end of the list
        else
        {
            boolean bFound = false;
            for ( int i = 0; i < sCriteria.length; i++ )
            {
                if ( sCriteria[i].startsWith( sURLInput.substring( 0, 4 ) ) )
                {//minimum size of a column name

                    arraySortOrders.set( i, sURLInput );
                    bFound = true;
                    break;
                }
            }
            if ( !bFound )
                arraySortOrders.add( sURLInput );
        }

        //force FileLister to reload the index (if necessary, after having set the archive mode
        //switch to false (to avoid reloading the file system)
        bIsArchiveMode = true;
        if ( fileLister.getResultCount() > 1 )
            fileLister.reloadIndex();
    }

    /**
     * Returns the descriptor of the chosen sort order, as to be shown on the results
     * site.
     *
     * @return   the sort order string
     */
    private String getSortDescriptor()
    {
        StringBuffer sb = new StringBuffer();
        String sNextEntry;
        for ( int i = 0; i < arraySortOrders.size(); i++ )
        {
            sb.append( ( i != 0 ) ? "/" : "" );
            sNextEntry = ( String )arraySortOrders.get( i );
            sb.append( ( sNextEntry.endsWith( SORTORDERS[0] )
                    ? sNextEntry.substring( 0, sNextEntry.length() - SORTORDERS[0].length() )
                    : sNextEntry.substring( 0, sNextEntry.length() - SORTORDERS[1].length() ) + " " + SORTORDERS[1] ) );
        }
        return sb.toString();
    }

    /**
     * Returns the descriptor of the search string(s), as to be shown on the results
     * site.
     *
     * @return   the search string(s)
     */
    private String getSearchStringDescriptor()
    {
        String sReturnValue = "";
        String sNext;
        for ( int i = 0; i < getSearchParameters().size(); i++ )
        {
            if ( i != 0 )
                sReturnValue += " / ";
            sNext = ( ( SearchParameters )getSearchParameters().get( i ) ).getSearchString();
            sReturnValue += ( StringUtils.isEmpty( sNext ) ) ? "(none)" : sNext;
        }

        return sReturnValue;
    }

    /**
     * Returns the last SearchParameters instance in the internal list or null, if
     * none.
     *
     * @return   the last stored SearchParameters or null
     */
    private SearchParameters getLastSearchParam()
    {
        if ( arraySearchParams.size() == 0 )
            return null;

        return ( SearchParameters )arraySearchParams.get( arraySearchParams.size() - 1 );
    }

    /**
     * Calculates the start index of the first visible entry of the last visible
     * site of a result set.
     *
     * @return   the start index of the last site
     */
    private int getLastVisibleIndex()
    {
        int iLastVisibleIndex = getStartIndex() + getResultsPerSite();
        if ( iLastVisibleIndex >= getFileLister().getResultCount() )
            iLastVisibleIndex = getFileLister().getResultCount() - 1;
        return iLastVisibleIndex;
    }

    /**
     * Returns the descriptor of the chosen min/max size, as to be shown on the results
     * site.
     *
     * @param params  the fully configured SearchParameters instance
     * @return        the min/max size string
     */
    private String getSizeDescriptor( SearchParameters params )
    {
        StringBuffer sb = new StringBuffer();
        if ( params.getMinimumSize() > SearchParameters.NOTSPECIFIED )
        {
            if ( params.getMaximumSize() <= SearchParameters.NOTSPECIFIED )
                sb.append( ">= " );
            sb.append( StringUtils.formatSize( params.getMinimumSize() * params.getMinimumMultiplier() ) );
            if ( params.getMaximumSize() > SearchParameters.NOTSPECIFIED )
                sb.append( " - " );
        }
        if ( params.getMaximumSize() > SearchParameters.NOTSPECIFIED )
        {
            if ( params.getMinimumSize() <= SearchParameters.NOTSPECIFIED )
                sb.append( "<= " );
            sb.append( StringUtils.formatSize( params.getMaximumSize() * params.getMaximumMultiplier() ) );
        }
        return sb.toString();
    }

    /**
     * Returns the descriptor of the chosen start/end date, as to be shown on the
     * results site.
     *
     * @param params  the fully configured SearchParameters instance
     * @return        the start/end date string
     */
    private String getDateDescriptor( SearchParameters params )
    {
        StringBuffer sb = new StringBuffer();
        if ( params.getStartingDate() != null )
        {
            if ( params.getEndingDate() == null )
                sb.append( ">= " );
            sb.append( DateFormat.getDateInstance( DateFormat.SHORT ).format( params.getStartingDate() ) );
            if ( params.getEndingDate() != null )
                sb.append( " - " );
        }
        if ( params.getEndingDate() != null )
        {
            if ( params.getSearchString() == null )
                sb.append( "<= " );
            sb.append( DateFormat.getDateInstance( DateFormat.SHORT ).format( params.getEndingDate() ) );
        }
        return sb.toString();
    }

    /**
     * Adds the default sort order(s) to the list of sort orders, assuming that it's
     * empty and there is a need for a new / the default sort order.
     */
    private void addDefaultSortOrder()
    {
        arraySortOrders.add( DEFAULTSORTORDER );//name, ascending
    }

    /**
     * Appends "?window=" + getWindowID zu the given string, for usage in URL's.
     *
     * @param sToWhat  the URL to append the window URL parameter to
     * @return         the updated URL target
     * @see            #getWindowID
     */
    private String appendWindowID( String sToWhat )
    {
        return new StringBuffer( sToWhat ).append( "?window=" ).append( getWindowID() ).toString();
    }

    /**
     * Returns the current search mode defining int.
     *
     * @return   NEWSEARCH or SEARCHWITHINRESULTS or SEARCHAGAIN, depending on the
     *      return value of the getSearchType() method
     */
    private int getSearchMode()
    {
        int iSearchMode = NEWSEARCH;
        if ( !StringUtils.isEmpty( getSearchType() ) )
        {
            if ( getSearchType().equals( "withinresults" ) )
                iSearchMode = SEARCHWITHINRESULTS;
            else if ( getSearchType().equals( "searchagain" ) )
                iSearchMode = SEARCHAGAIN;
        }
        return iSearchMode;
    }
}
