package org.alltimeflashdreamer.filelister;

import java.io.IOException;
import java.util.ArrayList;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.alltimeflashdreamer.util.StringUtils;

/**
 * <p>
 *
 * Title: FileListerDownloadServlet</p> <p>
 *
 * Description: This servlet controls the download-zip-file process, including the
 * generation and the streaming of the zip file and the producing of text output
 * for the client while waiting until the file is created.</p> <p>
 *
 * Organization: All Time Flash Dreamer</p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */

public class FileListerDownloadServlet extends AbstractFileListerServlet
{
    /**
     * Reads the download parameter values from the URL (which are just indexes within
     * the FileLister list) and packs the wished/clicked files into a zip which is
     * then sent to the client, together with the right MIME encoding. Because the
     * generation of such a zip fail may take quite a while, it's done by the ZipBuilderThread
     * class which is required on a periodic base (the intermediarily shown download
     * site has a meta-refresh tag which always recalls this servlet). As long as
     * the thread hasn't finished, the download site is shown again, and in the end,
     * the zip file is streamed to the client.
     *
     * @param request            the request object to read URL stuff from
     * @param response           the response object to send stuff to
     * @throws IOException       as defined in the superclass
     */
    public void doPost( HttpServletRequest request, HttpServletResponse response )
        throws IOException
    {
        //get the magic config object; if missing, redirect to app root
        FileListerConfig fileListerConfig = FileListerFactory.getFileListerConfig( request );
        if ( fileListerConfig == null )
        {
            FileListerFactory.redirectToAppRoot( response );
            return;
        }

        //check whether an error must be handled which occurred on the last
        //run of this servlet; if so unset the ZipBuilderThread pointer,
        //indicating that the zip building process was canceled
        if ( fileListerConfig.getFileListerException() != null )
        {
            fileListerConfig.setZipBuilderThread( null );
            response.sendRedirect( fileListerConfig.getErrorURL() );
            return;
        }

        //check whether the selectall or deselectall option is on; if so, process
        //this action and return to the viewer servlet
        String sZipMode = request.getParameter( "zipmode" );
        if ( !StringUtils.isEmpty( sZipMode ) )
        {
            fileListerConfig.getFileLister().processMultiSelectForZip( sZipMode );
            response.sendRedirect( fileListerConfig.getViewerServletURL() );
            return;
        }

        //check whether a ZipBuilderThread is currently set; if so, something's got
        //to be done with it: wait again, stream the finished zip file or unset the
        //pointer
        if ( fileListerConfig.getZipBuilderThread() != null )
        {
            if ( fileListerConfig.getZipBuilderThread().hasHundredPercentPageBeenShown() )
                streamZipFile( response, fileListerConfig );
            else if ( fileListerConfig.isZipBuilding() || fileListerConfig.getZipBuilderThread().isReadyToStream() )
                showCommentPage( response, fileListerConfig );
            else
            {
                fileListerConfig.setZipBuilderThread( null );
                response.sendRedirect( fileListerConfig.getRequestURL( false ) );
            }
            return;
        }

        //read the "download" URL parameter values and add the newly selected indexes
        if ( !readDownloadParameters( request, response, fileListerConfig ) )
            return;

        //get the list of selected indexes to be zipped or abort/redirect, if nothing to do
        ArrayList arrayFileListerEntries = fileListerConfig.getFileLister().getFileListerEntriesForZip();
        if ( arrayFileListerEntries.size() == 0 )
        {
            fileListerConfig.setFileListerException( new FileListerException( 42 ) );
            response.sendRedirect( fileListerConfig.getErrorURL() );
            return;
        }

        //remove duplicates (i.e. children of others)
        removeDuplicateEntries( arrayFileListerEntries );

        //create and start the ZipBuilderThread and redirect the client to the "wait for download" site
        try
        {
            ZipBuilderThread zbt = new ZipBuilderThread( fileListerConfig, arrayFileListerEntries );
            fileListerConfig.setZipBuilderThread( zbt );
            zbt.start();
            response.sendRedirect( fileListerConfig.getDownloadSite() );
        } catch ( FileListerException fle )
        {
            fileListerConfig.setFileListerException( fle );
            response.sendRedirect( fileListerConfig.getErrorURL() );
            return;
        }
    }

    /**
     * Scans the list for entries having a node id that indicates that they're a
     * child of another FileListerEntry in the given list. Such a child's node id
     * would look like the one of another entry, followed by a "0" and something
     * else.
     *
     * @param arrayFileListerEntries  the list of selected entries
     */
    private void removeDuplicateEntries( ArrayList arrayFileListerEntries )
    {
        FileListerEntry nextFLE;
        FileListerEntry nextCheckedFLE;
        ArrayList arrayToBeRemoved = new ArrayList();
        for ( int i = 0; i < arrayFileListerEntries.size(); i++ )
        {
            nextFLE = ( FileListerEntry )arrayFileListerEntries.get( i );
            for ( int j = 0; j < arrayFileListerEntries.size(); j++ )
            {
                if ( i == j )
                    continue;
                nextCheckedFLE = ( FileListerEntry )arrayFileListerEntries.get( j );
                if ( nextCheckedFLE.getNodeID().startsWith( nextFLE.getNodeID() + "0" ) )
                {
                    //found child: add it to the list of FileListerEntries to be removed
                    arrayToBeRemoved.add( nextCheckedFLE );
                }
            }
        }

        //remove those to be removed from the list
        for ( int i = 0; i < arrayToBeRemoved.size(); i++ )
            arrayFileListerEntries.remove( arrayToBeRemoved.get( i ) );
    }

    /**
     * Reads the list of request parameter values from the "download" key. If there
     * are such values, the parsed into int and set to the given FileListerConfig
     * instance.
     *
     * @param request           the request to read the parameters from
     * @param response          the response to redirect the client to the error
     *      page, if necessary
     * @param fileListerConfig  the magic config object
     * @return                  whether there occurred an exception while parsing
     *      the input or not
     * @throws IOException      thrown on redirecting the client
     */
    private boolean readDownloadParameters( HttpServletRequest request,
            HttpServletResponse response, FileListerConfig fileListerConfig )
        throws IOException
    {
        String[] sDownloadIndexes = request.getParameterValues( "download" );
        if ( sDownloadIndexes != null )
        {
            try
            {
                int[] iDownloadIndexes = new int[sDownloadIndexes.length];
                for ( int i = 0; i < iDownloadIndexes.length; i++ )
                    iDownloadIndexes[i] = new Integer( sDownloadIndexes[i] ).intValue();
                fileListerConfig.setSelectedIndexesForZip( iDownloadIndexes );
            } catch ( NumberFormatException nfe )
            {
                fileListerConfig.setFileListerException( new FileListerException( 35, nfe ) );
                response.sendRedirect( fileListerConfig.getErrorURL() );
                return false;
            }
        }
        return true;
    }

    /**
     * Waits a certain time and asks the FileListerConfig's ZipBuilderThread during
     * this waiting whether it's finished or not. If the thread still hasn't finished
     * after a certain time, it redirects the client to the selfredirecting download
     * site.
     *
     * @param response           the connection back to the client
     * @param fileListerConfig   the magic config object
     * @throws IOException       <i>only</i> if the redirect to the error site threw
     *      an IOException, i.e. within the stream method's own catch block
     */
    private void showCommentPage( HttpServletResponse response, FileListerConfig fileListerConfig )
        throws IOException
    {
        int iLoopCtr = 0;
        boolean bIsReadyToStream = false;
        while ( !( bIsReadyToStream = fileListerConfig.isReadyToStream() ) && iLoopCtr < 40 )
        {
            try
            {
                Thread.sleep( 50 );
                iLoopCtr++;
            } catch ( InterruptedException ie )
            {
                //bad dreams?
            }
        }

        //show the download site that corresponds to bIsReadyToStream: if true, the zip building is
        //finished and the "is completed" site is shown, else the "still in progress" site is shown
        response.sendRedirect( fileListerConfig.getDownloadSite() );
        if ( bIsReadyToStream )
            fileListerConfig.getZipBuilderThread().setHundredPercentPageWasShown();
    }

    /**
     * Tells the ZipBuilderThread (which is stored in the given FileListerConfig
     * object) to stream the generated zip file to the response's output stream.
     *
     * @param response          where to stream the zip file to
     * @param fileListerConfig  where the ZipBuilderThread is stored
     * @throws IOException      <i>only</i> if the redirect to the error site threw
     *      an IOException, i.e. within the stream method's own catch block
     */
    private void streamZipFile( HttpServletResponse response, FileListerConfig fileListerConfig )
        throws IOException
    {
        fileListerConfig.getZipBuilderThread().streamZipFile( response );
        fileListerConfig.setZipBuilderThread( null );
    }
}
