package org.alltimeflashdreamer.filelister;

import java.io.*;
import org.alltimeflashdreamer.util.*;

/**
 * <p>
 *
 * Title: FileListerEntry</p> <p>
 *
 * Description: Very simple value class to store the attributes of one found and
 * accepted file or directory.</p> <p>
 *
 * Organization: All Time Flash Dreamer</p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */

public class FileListerEntry implements Serializable, Chainable
{
    private String sName;
    private String sDirectory;
    private long lSizeInBytes;
    private long lLastModified;
    private String sType;
    private long lLastIndexed;
    private String sNodeID;
    private Chainable nextEntry;
    private boolean bIsSelectedForZip = false;

    /**
     * Reads the settings from a File instance.<br>
     * Note: On special circumstances, the getName method of the File class returns
     * null. If so, the name is set to the file's absolute path.
     *
     * @param file          the previously found/accepted File
     * @param lLastIndexed  the timestamp when the file was found
     * @param sNodeID       the node if of the file, as defined by the building of
     *      the binary tree down from the virtual root directory
     */
    public FileListerEntry( File file, long lLastIndexed, String sNodeID )
    {
        this( ( StringUtils.isEmpty( file.getName() ) ) ? file.getAbsolutePath() : file.getName(),
                file.getParent(), file.length(), file.lastModified(),
                ( file.isFile() ) ? "F" : "D", lLastIndexed, sNodeID );
    }

    /**
     * Initializes this class with each value.
     *
     * @param sName          the name of the entry
     * @param sDirectory     the directory of the entry
     * @param lSizeInBytes   the size of the file
     * @param lLastModified  the last modification date of the file
     * @param sType          the type descriptor ("F" for file or "D" for directory)
     * @param lLastIndexed   the timestamp when the file was found
     * @param sNodeID        the node if of the file, as defined by the building
     *      of the binary tree down from the virtual root directory
     */
    public FileListerEntry( String sName, String sDirectory,
            long lSizeInBytes, long lLastModified,
            String sType, long lLastIndexed, String sNodeID )
    {
        if ( FileLister.ISWINDOWS )
        {
            this.sName = StringUtils.replaceBackslashes( sName );
            this.sDirectory = StringUtils.replaceBackslashes( sDirectory );
        }
        else
        {
            this.sName = sName;
            this.sDirectory = sDirectory;
        }
        if ( this.sName.endsWith( "/" ) )
            this.sName = this.sName.substring( 0, this.sName.length() - 1 );

        this.lSizeInBytes = lSizeInBytes;
        this.lLastModified = lLastModified;
        this.sType = sType;
        this.lLastIndexed = lLastIndexed;
        this.sNodeID = sNodeID;
    }

    /**
     * Returns the name setting.
     *
     * @return   the name of this entry
     */
    public String getName()
    {
        return sName;
    }

    /**
     * Returns the directory setting.
     *
     * @return   the directory name of this entry
     */
    public String getDirectory()
    {
        return sDirectory;
    }

    /**
     * Returns the size (in bytes) of this entry.
     *
     * @return   the size of this entry
     */
    public long getSize()
    {
        return lSizeInBytes;
    }

    /**
     * Sets the size (in bytes) of this entry.
     *
     * @param lSizeInBytes  the size of this entry
     */
    public void setSize( long lSizeInBytes )
    {
        this.lSizeInBytes = lSizeInBytes;
    }

    /**
     * Returns the last modification date of this entry, as a long.
     *
     * @return   the last modification date of this entry
     */
    public long getLastModified()
    {
        return lLastModified;
    }

    /**
     * Returns whether this entry represents a file or not.
     *
     * @return   a file?
     */
    public boolean isFile()
    {
        return sType.equals( "F" );
    }

    /**
     * Returns whether this entry represents a directory or not.
     *
     * @return   a directory?
     */
    public boolean isDirectory()
    {
        return sType.equals( "D" );
    }

    /**
     * Returns the type-specifing string (either "D" for directory or "F" for file).
     *
     * @return   the value that represents the type of the entry in the database
     */
    public String getType()
    {
        return sType;
    }

    /**
     * Returns the time when the file was found/indexed, as a long.
     *
     * @return   the timestamp when the file was found
     */
    public long getLastIndexed()
    {
        return lLastIndexed;
    }

    /**
     * Returns the node id of this entry, as defined by the building of the binary
     * tree down from the virtual root directory.
     *
     * @return   this entry's node id
     */
    public String getNodeID()
    {
        return sNodeID;
    }

    /**
     * Sets the pointer to the next file lister entry in the list.
     *
     * @param nextEntry  the next, preinstantiated FileListerEntry; if null, the
     *      this element becomes the last one of the list
     */
    public void setNextEntry( Chainable nextEntry )
    {
        this.nextEntry = nextEntry;
    }

    /**
     * Returns the pointer to the next entry of the list, if any.
     *
     * @return   the next entry or null
     */
    public Chainable getNextEntry()
    {
        return nextEntry;
    }

    /**
     * Tells whether the user selected this entry for download in a zip.
     *
     * @return   to be added to a download zip?
     */
    public boolean isSelectedForZip()
    {
        return this.bIsSelectedForZip;
    }

    /**
     * Sets this entry as selected to be downloaded in a zip.
     *
     * @param bIsSelectedForZip  to be added to a download zip?
     */
    public void setSelectedForZip( boolean bIsSelectedForZip )
    {
        this.bIsSelectedForZip = bIsSelectedForZip;
    }
}
