package org.alltimeflashdreamer.filelister;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.alltimeflashdreamer.filelister.xml.FileListerXMLHandler;
import org.alltimeflashdreamer.util.StringUtils;

/**
 * <p>
 *
 * Title: FileListerFactory</p> <p>
 *
 * Description: This class is designed to be used by the various JSP sites, to avoid
 * having too much code within those sites. The public methods serve the JSP sites
 * directly, using a simple, static, one-line call.</p> <p>
 *
 * Organization: All Time Flash Dreamer</p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */

public final class FileListerFactory
{
    /**
     * Does the jobs to be done before the result site's frame content sites can
     * show the results/configuration appropriately:<br>
     * - reads the configuration file<br>
     * - verifies that the connection pool is initialized<br>
     * - verifies that a FileListerConfig instance is initialized and stored within
     * the session, together with a unique window id (which is stored in the new
     * FileListerConfig instance) - return the new instance for immediate work
     *
     * @param request               the request of the "JSP servlet"
     * @param response              the response of the "JSP servlet"
     * @param context               the servlet context of the "JSP servlet"
     * @return                      the newly created FileListerConfig, ready to
     *      work with it
     * @throws FileListerException  thrown only if the FileListerConfig wasn't stored
     *      in the user's session
     * @throws IOException          if the redirect to the error site somehow failed
     */
    public static FileListerConfig initialize( HttpServletRequest request,
            HttpServletResponse response, ServletContext context )
        throws FileListerException, IOException
    {
        FileListerConfig fileListerConfig = null;

        try
        {
            //read the "filelister.configdir" init parameter and test relative dir path first, then absolute
            String sServletDir = StringUtils.appendFileSeparator( context.getRealPath( "." ), true );
            String sConfigDir = StringUtils.appendFileSeparator( context.getInitParameter( "filelister.configdir" ), true );

            File configFile = new File( sServletDir + sConfigDir + FileListerXMLHandler.CONFIGFILE );
            if ( !configFile.exists() )
                configFile = new File( sConfigDir + FileListerXMLHandler.CONFIGFILE );
            if ( !configFile.exists() || !configFile.canRead() )
                throw new FileListerException( 49 );

            //make sure the connection pool and the property file parameters are initialized
            PropertyFileParameters params = new PropertyFileParameters( StringUtils.appendFileSeparator( configFile.getAbsoluteFile().getParent(), true ) );
            PoolHandler.initializePool( params );

            //build the magic config object, set the query builder's pointer and
            //add the config object to the FileListerConfig list of the client's session
            ArrayList arrayFileListerConfigs = getFileListerConfigList( request );
            int iWindowID = arrayFileListerConfigs.size();
            fileListerConfig = new FileListerConfig( request, context, params, iWindowID );
            fileListerConfig.getQueryBuilder().setFileListerConfig( fileListerConfig );
            arrayFileListerConfigs.add( fileListerConfig );
            setFileListerConfigList( request, arrayFileListerConfigs );
        } catch ( FileListerException fle )
        {
            handleFileListerException( response, fileListerConfig, fle );
        }

        //return the new instance (if come to here, it's been creatable...)
        return fileListerConfig;
    }

    /**
     * Processes the task of executing a search within the index or the file system
     * by reading the URL parameters and reacting according to them.<br>
     * There are several possibilities/scenarios what's to be processed:<br>
     * - Click on a link within a column header<br>
     * - Click on the "jump-to-site" button<br>
     * - Click on the "Show results:" button<br>
     * - Click on a navigation button (first, previous, next, last)<br>
     * Everything but the column header link click is done within the optionbar site,
     * so those URL inputs are handled <i>together</i> , which means the user may
     * change the number of visible entries and the shown site together.
     *
     * @param request               the request of the "JSP servlet"
     * @param response              the response of the "JSP servlet"
     * @throws FileListerException  if the processing of the URL input or the search
     *      itself threw an exception which couldn't be shown to the client
     * @throws IOException          if the redirect to the error site somehow failed
     */
    public static void processSearch( HttpServletRequest request, HttpServletResponse response )
        throws FileListerException, IOException
    {
        FileListerConfig fileListerConfig = getFileListerConfig( request );

        try
        {
            //if it's the first load of a new search configuration, FileLister is
            //still null -> create it
            FileLister fileLister = fileListerConfig.getFileLister();
            if ( fileLister == null || !StringUtils.isEmpty( request.getParameter( "definesearch" ) ) )
                createFileLister( fileListerConfig, request );
            //else this is a reload of the same configuration
            //Note: the jump-to-site, the show-results and the nav buttons may be followed
            //by the URL parameter "selectedforzip", filled with JavaScript/onSubmit; this
            //parameter's value is a comma-separated list of indexes that are selected
            else
            {
                //DEBUG: printRequestParameters( request );

                if ( !StringUtils.isEmpty( request.getParameter( "column" ) ) )
                    fileListerConfig.sortByColumn( request.getParameter( "column" ) );
                else if ( !StringUtils.isEmpty( request.getParameter( "jumptosite" ) ) )
                {
                    //set the parameters anyway that are in the URL anyway, to avoid problems with
                    //different browsers (some send the input names on key events, some don't...)
                    addSelectedIndexesToZip( fileListerConfig, request.getParameter( "selectedforzip" ) );
                    fileListerConfig.jumpToSite( request.getParameter( "jumptosite" ) );
                    fileListerConfig.setResultsPerSite( request.getParameter( "resultscount" ) );

                    //search for a "nav" URL parameter and handle it, if any (NOTE: since the nav buttons
                    //are of type "image", they produce two additional URL parameters, one for the x and one
                    //for the y coordinates of the clicked image -> only read one of them!
                    Enumeration eNames = request.getParameterNames();
                    while ( eNames.hasMoreElements() )
                    {
                        String sNextName = ( String )eNames.nextElement();
                        if ( sNextName.startsWith( "nav" ) )
                        {
                            fileListerConfig.navigate( sNextName.substring( 3 ) );
                            break;
                        }
                    }
                }
                //in each other case, the user probably just reloaded to site -> don't do anything
            }
        } catch ( FileListerException fle )
        {
            handleFileListerException( response, fileListerConfig, fle );
        }
    }

    /**
     * If the given FileListerConfig isn't null, the catched exception is stored
     * therein and the client is redirected to the error site, else the exception
     * itself is throw again.
     *
     * @param response              the response to redirect the client to
     * @param fileListerConfig      the magic config object, ready to take the given
     *      exception
     * @param fle                   the exception to handle
     * @throws FileListerException  the given exception, in case the FileListerConfig
     *      is null
     * @throws IOException          in case the redirection failed somehow
     */
    public static void handleFileListerException( HttpServletResponse response,
            FileListerConfig fileListerConfig, FileListerException fle )
        throws FileListerException, IOException
    {
        if ( fileListerConfig != null )
        {
            fileListerConfig.setFileListerException( fle );
            response.sendRedirect( fileListerConfig.getRequestURL( false ) );
        }
        else
            throw fle;
    }

    /**
     * Gets the instance of FileListerConfig that is specified by the URL parameter
     * "window" from the session's attribute list. If no such URL parameter is specified,
     * this means that a new window was created and a new instance of FileListerConfig
     * must be created.
     *
     * @param request  the request to get an attribute of
     * @return         the specified FileListerConfig or null, it not stored
     */
    public static FileListerConfig getFileListerConfig( HttpServletRequest request )
    {
        try
        {
            return ( FileListerConfig )getFileListerConfigList( request ).get(
                    new Integer( request.getParameter( "window" ) ).intValue() );
        } catch ( Throwable t )
        {
            return null;
        }
    }

    /**
     * Returns the client to the application root in one of the many possible cases
     *
     * @param response
     * @throws IOException
     */
    public static void redirectToAppRoot( HttpServletResponse response )
        throws IOException
    {
        response.setContentType( "text/html" );
        response.sendRedirect( FileListerConfig.CONTROLLERSERVLET );
    }

    /**
     * Prints out all the parameter key/value pairs to the standard out.
     *
     * @param request  the request to read the parameters from
     */
    public static void printRequestParameters( HttpServletRequest request )
    {
        Enumeration eKeys = request.getParameterNames();
        while ( eKeys.hasMoreElements() )
        {
            String sNextKey = ( String )eKeys.nextElement();
            String[] sNextValues = request.getParameterValues( sNextKey );
            for ( int i = 0; sNextValues != null && i < sNextValues.length; i++ )
            {
                System.out.println( sNextKey + " = " + sNextValues[i] );
            }
        }
    }

//---------------------------- private methods -------------------------------------//
    /**
     * (Re)Initializes a FileLister for the given FileListerConfig, after processing
     * the search parameters stored in the given request object.
     *
     * @param fileListerConfig      the owner of the FileLister to be created
     * @param request               the request to get the search parameters from
     * @throws FileListerException  if some input is invalid or the FileLister couldn't
     *      be created
     */
    private static void createFileLister( FileListerConfig fileListerConfig, HttpServletRequest request )
        throws FileListerException
    {
        //read the URL parameters into a SearchParameters object
        fileListerConfig.createSearchParameters( request );

        //get the FileLister pointer; if already existent, delete it
        FileLister fileLister = fileListerConfig.getFileLister();
        if ( fileLister != null )
        {
            fileLister = null;
            fileListerConfig.setFileLister( null );

            //sometimes, a previous, very large FileLister may lead to an OutOfMemoryError...
            System.gc();
        }

        //initialize the new FileLister and set it to FileListerConfig
        fileLister = new FileLister( fileListerConfig, false );
        fileListerConfig.setFileLister( fileLister );
    }

    /**
     * Parses the comma-separated list of indexes in sURLInput, if not empty, builds
     * an int[] out of it and passes this one to FileListerConfig's setSelectedForZip
     * method.
     *
     * @param fileListerConfig      the magic config object
     * @param sURLInput             the URL input value of the "selectedforzip" parameter
     * @throws FileListerException  if the URL input is not parseable into an int[]
     */
    private static void addSelectedIndexesToZip( FileListerConfig fileListerConfig, String sURLInput )
        throws FileListerException
    {
        if ( StringUtils.isEmpty( sURLInput ) )
        {
            fileListerConfig.setSelectedIndexesForZip( null );
            return;
        }

        StringTokenizer st = new StringTokenizer( sURLInput, "," );
        try
        {
            int[] iSelectedIndexes = new int[st.countTokens()];
            int iCtr = 0;
            while ( st.hasMoreElements() )
            {
                iSelectedIndexes[iCtr] = new Integer( st.nextToken() ).intValue();
                iCtr++;
            }
            //notify FileListerConfig about the list of selected indexes
            fileListerConfig.setSelectedIndexesForZip( iSelectedIndexes );
        } catch ( Exception ex )
        {
            throw new FileListerException( 4, ex );
        }
    }

    /**
     * Gets the ArrayList of FileListerConfig objects from the session and returns
     * it, if already initialized; else a new one is initialized and returned.
     *
     * @param request  the user's request
     * @return         the list of the session or a new one
     */
    private static ArrayList getFileListerConfigList( HttpServletRequest request )
    {
        ArrayList returnValue = ( ArrayList )request.getSession( true ).getAttribute( FileListerViewerServlet.CONFIGCLASSNAME );
        if ( returnValue == null )
            returnValue = new ArrayList();
        return returnValue;
    }

    /**
     * Sets the given list to the given request's attribute list.
     *
     * @param request  the request to set the list to
     * @param list     the list to set to the request
     */
    private static void setFileListerConfigList( HttpServletRequest request, ArrayList list )
    {
        request.getSession().setAttribute( FileListerViewerServlet.CONFIGCLASSNAME, list );
    }
}
