package org.alltimeflashdreamer.filelister;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;

import org.alltimeflashdreamer.util.zip.InvalidDirectoryException;
import org.alltimeflashdreamer.util.zip.ZipBuilder;

/**
 * <p>
 *
 * Title: ZipBuilder</p> <p>
 *
 * Description: Builds a zip file out of the source received within the constructor.
 * This source is either a list of files/directories, together with a target output
 * stream or just a directory string (two different constructors). The first one
 * is to be used from the web GUI, the second from the command line.</p> <p>
 *
 * Organization: All Time Flash Dreamer</p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */

class FileListerZipBuilder extends ZipBuilder
{
    /**
     * Creates a new zip file in the nominated temp directory of the config class.
     * The name of this new file is stored for later returning through the getZipFileName
     * method.
     *
     * @param fileListerConfig      the often used config class
     * @param os                    the output stream to connect the ZipOutputStream to
     * @throws FileListerException  wrapper for each possible "foreseen" exception
     */
    FileListerZipBuilder( FileListerConfig fileListerConfig,
            OutputStream os )
        throws FileListerException
    {
        super( os, fileListerConfig.getZipLevel() );

        if ( fileListerConfig == null || os == null )
            throw new FileListerException( 38 );
    }

    /**
     * Initializer method for the access of this class by the ZipBuilderThread. It
     * calls the addFileToZip and the recurseAndZipDirectory methods for each entry
     * of the given list of files and directories.
     *
     * @param fileOrDirList         list of File objects representing the files and
     *      directories that shall be added to the new zip file
     * @param fileListerConfig      the magic config object
     * @throws FileListerException  if something went wrong with the zip building
     */
    void buildZip( File[] fileOrDirList, FileListerConfig fileListerConfig )
        throws FileListerException
    {
        if ( fileOrDirList == null || fileOrDirList.length == 0 ||
                fileListerConfig == null )
            throw new FileListerException( 38 );

        //Because the virtual root internally contains only forward slashes as file separators,
        //it must be converted to system specific separators (in case of Windows only)
        String sSuppressedPrefix = fileListerConfig.getVirtualRoot();
        if ( FileLister.ISWINDOWS )
            sSuppressedPrefix = sSuppressedPrefix.replace( '/', '\\' );

        try
        {
            //pack the directory's content into the zip output stream but only if the
            //entries exist
            for ( int i = 0; i < fileOrDirList.length; i++ )
            {
                if ( !fileOrDirList[i].exists() || !fileOrDirList[i].canRead() )
                    continue;
                if ( fileOrDirList[i].isFile() )
                    addFileToZip( fileOrDirList[i], sSuppressedPrefix );
                else if ( fileOrDirList[i].isDirectory() )
                    recurseAndZipDirectory( fileOrDirList[i], sSuppressedPrefix, "" );
            }
        } catch ( InvalidDirectoryException ide )
        {
            throw new FileListerException( 36, ide );
        } catch ( IOException ioe )
        {
            throw new FileListerException( 37, ioe );
        }

        try
        {
            closeOutputStreams();
        } catch ( IOException ioe )
        {
            throw new FileListerException( 39, ioe );
        }
    }
}
