package org.alltimeflashdreamer.filelister;

import java.io.*;
import java.util.*;
import org.alltimeflashdreamer.filelister.xml.*;
import org.alltimeflashdreamer.util.StringUtils;
import org.xml.sax.SAXException;

/**
 * <p>
 *
 * Title: PropertyFileParameters</p> <p>
 *
 * Description: This class stores the values of the property file, for easy handling
 * by the FileListerConfig and the PoolHandler/ConnectionPool classes.</p> <p>
 *
 * Organization: All Time Flash Dreamer</p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */
public class PropertyFileParameters implements Serializable
{
    private final static int DEFAULTPACKAGINGSTRENGTH = 4;

    /** "directory.virtualroot" parameter value */
    String sVirtualRootDirectory = "";
    /** "directory.showabsolutepaths" parameter value (true is default) */
    private boolean bShowAbsolutePaths = true;
    /** "index.aging" parameter value */
    private int iIndexAging;
    /** "system.maxfilelisterobjects" parameter value */
    private int iMaxFileListerObjects;
    /** "directory.nogoarea" parameter values, tokenized and stored in a LinkedList */
    private LinkedList noGoAreas = new LinkedList();
    /** "jdbc.driverclass" parameter value */
    private String sDriverClassName;
    /** "jdbc.connectionstring" parameter value */
    private String sConnectionString;
    /** "jdbc.username" parameter value */
    private String sUserName;
    /** "jdbc.password" parameter value */
    private String sPassword;
    /** "jdbc.initialconnectionpoolsize" parameter value */
    int iInitialConnections;
    /** "jdbc.pooledconnectionincrement" parameter value */
    int iConnectionIncrement;
    /** "jdbc.tablename" parameter value */
    private String sJDBCTableName;
    /** "jdbc.querybuilderclass" parameter value, loaded and therefore verified */
    private SQLQueryBuilder sqlQueryBuilder;
    /** "link.prefix" parameter value */
    private String sLinkPrefix;
    /** "link.regexphelp" parameter value */
    private String sRegExpHelpLink;
    /** "zip.isallowedtozip" parameter value (false is default) */
    private boolean bIsAllowedToZip = false;
    /** "zip.creationdirectory" parameter value */
    private String sZipCreationDirectory;
    /** "zip.packagestrength" parameter value */
    private int iZipPackageStrength;

    //attention: FileListerXMLHandler is not serializable, therefore the save time and
    //the path of the underlying file must be stored too...
    private FileListerXMLHandler xmlHandler;
    private String sXMLFilePath;
    private long lLastModified;

    /**
     * Constructor parses the given config XML file and verifies the entries. If
     * an exception other than an IOException occurs, it's wrapped within a FileListerException
     * and thrown.
     *
     * @param sPropertyFileDir      the directory where the property file is stored
     * @throws FileListerException  each possible exception that may occur during
     *      the parsing-of-the-property-file process
     * @throws IOException          if the property file can't be accessed somehow
     */
    public PropertyFileParameters( String sPropertyFileDir )
        throws FileListerException, IOException
    {
        //build the full config file path (check whether it already ends with the config file name!)
        if ( sPropertyFileDir.endsWith( FileListerXMLHandler.CONFIGFILE ) )
            sPropertyFileDir = sPropertyFileDir.substring( 0, sPropertyFileDir.length() - FileListerXMLHandler.CONFIGFILE.length() );
        sXMLFilePath = StringUtils.appendFileSeparator( sPropertyFileDir, true ) + FileListerXMLHandler.CONFIGFILE;
        File xmlFile = new File( sXMLFilePath );
        sXMLFilePath = FileLister.getRealPath( xmlFile );

        //store the last-modified value, parse the config file and load/verify its entries
        lLastModified = xmlFile.lastModified();
        try
        {
            xmlHandler = new FileListerXMLHandler( xmlFile, false );
        } catch ( SAXException saxe )
        {
            throw new FileListerException( 47, saxe );
        }
        readEntries();
    }

    /**
     * Overloaded constructor which avoids the parsing of the file. Instead of this,
     * a fully configured reader is passed and the settings are read/validated from
     * of the given instance.
     *
     * @param xmlHandler            the previously instantiated config reader
     * @throws FileListerException  if there is an inacceptable config setting
     */
    public PropertyFileParameters( FileListerXMLHandler xmlHandler )
        throws FileListerException
    {
        this.xmlHandler = xmlHandler;
        try
        {
            sXMLFilePath = FileLister.getRealPath( xmlHandler.getFile() );
        } catch ( IOException ioe )
        {
            sXMLFilePath = xmlHandler.getFile().getPath();
        }
        readEntries();
    }

    /**
     * Returns the virtual root setting of the property file; this value may equal
     * the "", if not set.
     *
     * @param bWithReloadCheck  whether to check for a xml file reload or not
     * @return                  the "directory.virtualroot" parameter value
     */
    public String getVirtualRootDirectory( boolean bWithReloadCheck )
    {
        if ( bWithReloadCheck )
            checkReloadOfXMLFile();
        return sVirtualRootDirectory;
    }

    /**
     * Returns whether the file listing output shall contain absolute or virtual
     * paths within the link URL's.
     *
     * @return   the "directory.showabsolutepaths" parameter value (true is default)
     */
    public boolean isShowingAbsolutePaths()
    {
        checkReloadOfXMLFile();
        return bShowAbsolutePaths;
    }

    /**
     * Returns the amount of seconds that the engine shall wait until a directory
     * search is (physically) performed again.
     *
     * @param bWithReloadCheck  whether to check for a xml file reload or not
     * @return                  the "index.aging" parameter value
     */
    public int getIndexAging( boolean bWithReloadCheck )
    {
        if ( bWithReloadCheck )
            checkReloadOfXMLFile();
        return iIndexAging;
    }

    /**
     * Returns how many FileListerEntry objects shall be kept at most in the memory.
     *
     * @param bWithReloadCheck  whether to check for a xml file reload or not
     * @return                  the "system.maxfilelisterobjects" parameter value
     */
    public int getMaxFileListerObjects( boolean bWithReloadCheck )
    {
        if ( bWithReloadCheck )
            checkReloadOfXMLFile();
        return iMaxFileListerObjects;
    }

    /**
     * Returns a list of the tokenized entries of the no-go-areas, each defining
     * a path on the file system which must not be indexed.
     *
     * @param bWithReloadCheck  whether to check for a xml file reload or not
     * @return                  the "directory.nogoarea" parameter values, tokenized
     *      and stored in a LinkedList
     */
    public LinkedList getNoGoAreas( boolean bWithReloadCheck )
    {
        if ( bWithReloadCheck )
            checkReloadOfXMLFile();
        return noGoAreas;
    }

    /**
     * Returns the class name of the JDBC driver to be used by FileLister.
     *
     * @return   the "jdbc.driverclass" parameter value
     */
    public String getDriverClassName()
    {
        checkReloadOfXMLFile();
        return sDriverClassName;
    }

    /**
     * Returns the name of the connection string to be used by the JDBC driver used
     * by FileLister.
     *
     * @return   the "jdbc.connectionstring" parameter value
     */
    public String getConnectionString()
    {
        checkReloadOfXMLFile();
        return sConnectionString;
    }

    /**
     * Returns the user name of the database (used to build a connection).
     *
     * @return   the "jdbc.username" parameter value
     */
    public String getUserName()
    {
        checkReloadOfXMLFile();
        return sUserName;
    }

    /**
     * Returns the password of the database (used to build a connection).
     *
     * @return   the "jdbc.password" parameter value
     */
    public String getPassword()
    {
        checkReloadOfXMLFile();
        return sPassword;
    }

    /**
     * Returns the amount of connections that shall be established by the connection
     * pool on initialization.
     *
     * @param bWithReloadCheck  whether to check for a xml file reload or not
     * @return                  the "jdbc.initialconnectionpoolsize" parameter value
     */
    public int getInitialConnections( boolean bWithReloadCheck )
    {
        if ( bWithReloadCheck )
            checkReloadOfXMLFile();
        return iInitialConnections;
    }

    /**
     * Returns the amount of connections that shall be established by the connection
     * pool if the initial pool is not big enough (on heavy load).
     *
     * @param bWithReloadCheck  whether to check for a xml file reload or not
     * @return                  the "jdbc.pooledconnectionincrement" parameter value
     */
    public int getConnectionIncrement( boolean bWithReloadCheck )
    {
        if ( bWithReloadCheck )
            checkReloadOfXMLFile();
        return iConnectionIncrement;
    }

    /**
     * Returns the name of the table within the database where the index shall be
     * written into / read from.
     *
     * @return   the "jdbc.tablename" parameter value
     */
    public String getJDBCTableName()
    {
        checkReloadOfXMLFile();
        return sJDBCTableName;
    }

    /**
     * Returns the concrete implementation class of SQLQueryBuilder, adapted for
     * the database server to be used by the FileLister application.
     *
     * @return   "jdbc.querybuilderclass" parameter value, loaded into a subclass
     *      of SQLQueryBuilder and therefore verified
     */
    public SQLQueryBuilder getSQLQueryBuilder()
    {
        checkReloadOfXMLFile();
        return sqlQueryBuilder;
    }

    /**
     * Returns the prefix that shall be set before the path of each entry, so that
     * the entry can be viewed/downloaded from the web GUI (e.g. "ftp://localhost").
     *
     * @return   the "link.prefix" parameter value
     */
    public String getLinkPrefix()
    {
        checkReloadOfXMLFile();
        return sLinkPrefix;
    }

    /**
     * Returns the link that shall be used for explanation of the regexp functionality
     * in the "definesearch" site.
     *
     * @return   the "link.regexphelp" parameter value
     */
    public String getRegExpHelpLink()
    {
        checkReloadOfXMLFile();
        return sRegExpHelpLink;
    }

    /**
     * Returns whether it's allowed to download web GUI entries in a zip file created
     * by FileLister or not.
     *
     * @return   the "zip.isallowedtozip" parameter value (false is default)
     */
    public boolean isAllowedToZip()
    {
        checkReloadOfXMLFile();
        return bIsAllowedToZip;
    }

    /**
     * Returns the path of the directory where the zip files shall be generated in
     * (used only if it's allowed to zip).
     *
     * @param bWithReloadCheck  whether to check for a xml file reload or not
     * @return                  the "zip.creationdirectory" parameter value
     */
    public String getZipCreationDirectory( boolean bWithReloadCheck )
    {
        if ( bWithReloadCheck )
            checkReloadOfXMLFile();
        return sZipCreationDirectory;
    }

    /**
     * Returns the zip package strength, the value which indicates how hard the generated
     * zip files shall be minimized (in size).
     *
     * @param bWithReloadCheck  whether to check for a xml file reload or not
     * @return                  the "zip.packagestrength" parameter value (4 is default)
     */
    public int getZipPackageStrength( boolean bWithReloadCheck )
    {
        if ( bWithReloadCheck )
            checkReloadOfXMLFile();
        return iZipPackageStrength;
    }

//------------------------------- inner methods ------------------------------------------//
    /**
     * Reads and validates the whole configuration.
     *
     * @exception FileListerException  wraps anything
     */
    private void readEntries()
        throws FileListerException
    {
        sDriverClassName = xmlHandler.getJDBC().getPropertyValue( "driverclass" );
        sConnectionString = xmlHandler.getJDBC().getPropertyValue( "connectionstring" );
        sUserName = xmlHandler.getJDBC().getPropertyValue( "username" );
        sPassword = xmlHandler.getJDBC().getPropertyValue( "password" );

        //load and verify the index table name
        sJDBCTableName = xmlHandler.getJDBC().getPropertyValue( "tablename" );
        if ( StringUtils.isEmpty( sJDBCTableName ) )
            throw new FileListerException( 27 );

        //if the virtual root IS defined, check whether it exists
        loadAndVerifyVirtualRoot();

        //parse directory.nogoarea parameters into a linked list (incl. transformation
        //into absolute path names, but without check for existence)
        loadAndVerifyNoGoAreas();

        //load and verify the index aging parameter
        try
        {
            iIndexAging = new Integer( xmlHandler.getIndex().getPropertyValue( "aging" ) ).intValue();
            if ( iIndexAging <= 0 )
                iIndexAging = 0;
        } catch ( Exception ex )
        {
            throw new FileListerException( 13 );
        }

        //load and verify the initial connections count, the connection increment
        //count and the maximum file lister objects count
        try
        {
            iInitialConnections = new Integer( xmlHandler.getJDBC().getPropertyValue( "initialconnectionpoolsize" ) ).intValue();
            iConnectionIncrement = new Integer( xmlHandler.getJDBC().getPropertyValue( "pooledconnectionincrement" ) ).intValue();
            iMaxFileListerObjects = new Integer( xmlHandler.getSystem().getPropertyValue( "maxfilelisterobjects" ) ).intValue();
        } catch ( Exception ex )
        {
            throw new FileListerException( 18, ex );
        }
        if ( iInitialConnections <= 0 || iConnectionIncrement <= 0 )
            throw new FileListerException( 18 );

        //make sure iMaxFileListerObjects is at least FileListerConfig.MINIMUMENTRIESINMEMORY
        //AND dividible by this value
        int iTemp = ( iMaxFileListerObjects / FileListerConfig.MINIMUMENTRIESINMEMORY );
        iMaxFileListerObjects = ( ( iTemp <= 0 ) ? 1 : iTemp ) * FileListerConfig.MINIMUMENTRIESINMEMORY;

        //load query builder class name and initialize this class (runs a lot
        //of checks and avoids later exceptions)
        String sQueryBuilderClassName = xmlHandler.getJDBC().getPropertyValue( "querybuilderclass" );
        try
        {
            sqlQueryBuilder = ( SQLQueryBuilder )Class.forName( sQueryBuilderClassName ).newInstance();
        } catch ( Exception ex )
        {
            throw new FileListerException( 29, ex );
        }

        //store the link prefix and regexp help link
        sLinkPrefix = xmlHandler.getLink().getPropertyValue( "prefix" );
        sRegExpHelpLink = xmlHandler.getLink().getPropertyValue( "regexphelp" );

        //load, verify and store the zip parameters
        loadAndVerifyZipParameters();

        //unset pointer of not serializable XML handler
        xmlHandler = null;
    }

    /**
     * Reads the virtual root setting and verifies its existence. If existing, the
     * "show absolute paths" setting is read too.
     *
     * @throws FileListerException  if the specified virtual root directory doesn't
     *      exist
     */
    private void loadAndVerifyVirtualRoot()
        throws FileListerException
    {
        sVirtualRootDirectory = xmlHandler.getDirectory().getPropertyValue( "virtualroot" );
        if ( !StringUtils.isEmpty( sVirtualRootDirectory ) )
        {
            //check whether the (virtual) root directory exists
            File rootDirectory = new File( sVirtualRootDirectory );
            if ( !rootDirectory.exists() || !rootDirectory.isDirectory() )
                throw new FileListerException( 5 );

            try
            {
                //convert path to absolute one and append file separator, if necessary
                sVirtualRootDirectory = StringUtils.replaceBackslashes(
                        StringUtils.appendFileSeparator( FileLister.getRealPath( rootDirectory ), true ) );
            } catch ( IOException ioe )
            {
                throw new FileListerException( 5, ioe );
            }

            //check the showabsolutepaths parameter (which is ignored if no virtual root is defined)
            try
            {
                String sTemp = xmlHandler.getDirectory().getPropertyValue( "showabsolutepaths" );
                if ( !StringUtils.isEmpty( sTemp ) && sTemp.equalsIgnoreCase( "false" ) )
                    bShowAbsolutePaths = false;
            } catch ( Exception ex )
            {
            }
        }
    }

    /**
     * Reads the "no-go area" parameter, splits it up into the comma-separated tokens
     * and stores them into a list. No checks are done against whether it's a directory
     * or whether the directory exists. The reason why there is no such check is
     * the fact that some platforms show stupid "not accessible" message boxes on
     * accessing such an entry, especially if it's a floppy drive.
     *
     * @throws FileListerException  if one of the no-go areas doesn't exist or it's
     *      path can't be translated to the absolute form
     */
    private void loadAndVerifyNoGoAreas()
        throws FileListerException
    {
        String sNoGoArea = xmlHandler.getDirectory().getPropertyValue( "nogoarea" );
        if ( StringUtils.isEmpty( sNoGoArea ) )
            sNoGoArea = "";
        StringTokenizer st = new StringTokenizer( sNoGoArea, "," );
        File nextNoGoDir;
        while ( st.hasMoreTokens() )
        {
            try
            {
                nextNoGoDir = new File( st.nextToken().trim() );
                noGoAreas.add( FileLister.getRealPath( nextNoGoDir ) );
            } catch ( IOException ioe )
            {
                throw new FileListerException( 25, ioe );
            }
        }
    }

    /**
     * Loads the parameters starting with "zip.", i.e. whether it's allowed to download
     * zip files and which strength shall be used making such a zip file. If somehow
     * the package strength can't be read/parsed, the default value of 4 is used.
     *
     * @throws FileListerException  if it's allowed to zip and the zip creation directory
     *      is missing
     */
    private void loadAndVerifyZipParameters()
        throws FileListerException
    {
        try
        {
            String sIsAllowedToZip = xmlHandler.getZip().getPropertyValue( "isallowedtozip" );
            bIsAllowedToZip = ( ( !StringUtils.isEmpty( sIsAllowedToZip ) &&
                    sIsAllowedToZip.equalsIgnoreCase( "true" ) ) ? true : false );
        } catch ( Exception ex )
        {

        }
        if ( bIsAllowedToZip )
        {
            try
            {
                //zip creation directory: must exist and be a "writable" directory
                File zipDir = new File( xmlHandler.getZip().getPropertyValue( "creationdirectory" ) );
                if ( !zipDir.isDirectory() || !zipDir.exists() || !zipDir.canWrite() )
                    throw new FileListerException( 40 );

                sZipCreationDirectory = StringUtils.appendFileSeparator( FileLister.getRealPath( zipDir ), true );
            } catch ( Exception ex )
            {
                if ( ex instanceof FileListerException )
                    throw ( FileListerException )ex;

                throw new FileListerException( 40, ex );
            }
            try
            {
                //zip packaging strength: if invalid, take default
                iZipPackageStrength = new Integer( xmlHandler.getZip().getPropertyValue( "packagingstrength" ) ).intValue();
                if ( iZipPackageStrength < 0 || iZipPackageStrength > 9 )
                    iZipPackageStrength = DEFAULTPACKAGINGSTRENGTH;
            } catch ( Exception ex )
            {
                iZipPackageStrength = DEFAULTPACKAGINGSTRENGTH;
            }
        }
        else
        {
            sZipCreationDirectory = "";
            iZipPackageStrength = DEFAULTPACKAGINGSTRENGTH;
        }
    }

    /**
     * Runs the "is needed to reload settings" check. If this returns that the XML
     * file was modified since the last load, the reloading is <i>tried to be done
     * </i>. If this reloading somehow fails, the upcoming exception's stack trace
     * is printed, but the application continues running (don't interrupt it, i.e.
     * don't throw exceptions one each single access of an app parameter).
     */
    private void checkReloadOfXMLFile()
    {
        try
        {
            File xmlFile = new File( sXMLFilePath );
            if ( xmlFile.lastModified() != lLastModified )
            {
                xmlHandler = new FileListerXMLHandler( xmlFile, false );
                readEntries();
                lLastModified = xmlFile.lastModified();
            }
        } catch ( Exception ex )
        {
            ex.printStackTrace();
        }
    }
}
