package org.alltimeflashdreamer.filelister;

import java.io.Serializable;
import java.util.ArrayList;

/**
 * <p>
 *
 * Title: SQLQueryBuilder</p> <p>
 *
 * Description: This abstract class defines the methods required by FileLister to
 * get the wellformed SQL strings for the various database accesses. Implementing
 * classes must focus on one specific SQL database server.<br>
 * Important: Implementing classes need an argument-less constructor.</p> <p>
 *
 * Organization: All Time Flash Dreamer</p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */

public abstract class SQLQueryBuilder implements Serializable
{
    private FileListerConfig fileListerConfig;
    private int iLowerLimit = 0;
    private int iUpperLimit = 0;

    /**
     * Sets the config object to be used by the implementing class.
     *
     * @param config  the object which knows everything about the user input
     */
    public final void setFileListerConfig( FileListerConfig config )
    {
        this.fileListerConfig = config;
    }

    /**
     * Returns whether a FileListerConfig has already been set or not.
     *
     * @return   config already set?
     */
    protected final boolean hasFileListerConfig()
    {
        return ( fileListerConfig != null );
    }

    /**
     * Extending classes need to have access to the config object, therefore this
     * method is protected.
     *
     * @return   the config object or null, if not set
     */
    protected final FileListerConfig getFileListerConfig()
    {
        return fileListerConfig;
    }

    /**
     * Returns the whole list of SearchParameters instances, each for one (consecutive)
     * search configuration input.
     *
     * @return   the list of SearchParameters instances
     */
    protected final ArrayList getSearchParameters()
    {
        return getFileListerConfig().getSearchParameters();
    }

    /**
     * Returns the lower limit value (which is 0, if not yet set).
     *
     * @return   the result-count-limitating lower boundary
     */
    protected int getLowerLimit()
    {
        return iLowerLimit;
    }

    /**
     * Returns the upper limit value (which is 0, if not yet set).
     *
     * @return   the result-count-limitating upper boundary
     */
    protected int getUpperLimit()
    {
        return iUpperLimit;
    }

    /**
     * Builds and returns the SQL query string used to get the information specified
     * by the user. This query string needs to be as effective as possible to avoid
     * long loading times and memory overhead.<br>
     * The boolean indicates whether the result count only or the full table data
     * shall be returned using this method's SQL string.<br>
     * It's up to the implementing class to use the getLimitationString method to
     * limit the result count as configured in the property file.
     *
     * @param sNodeID          the id of the directory to be searched
     * @param bForResultCount  whether the SQL string leads to the returning of the
     *      result count only or to the returning of the full table data
     * @return                 the SQL string which reduces the amount of returned
     *      entries as much as possible
     */
    public abstract String getUserSearchString( String sNodeID, boolean bForResultCount );

    /**
     * Builds and returns the SQL query string used to build an indexing table within
     * the database where the index tables are stored. Because a new table is created
     * for each separate index, the name of the new table is passed as a method attribute.
     *
     * @param sTableName  the name of the index table to be created
     * @return            the fully qualified SQL string
     */
    public abstract String getCreateTableString( String sTableName );

    /**
     * Builds and returns the SQL query string used to list all the tables that are
     * somehow like (or match) the given one in the current database. This is often
     * a highly proprietary mechanism!
     *
     * @param sTableName  the name of the table to find the similar/equal ones
     * @return            the fully qualified SQL string to get those table names,
     *      if any
     */
    public abstract String getListAllTablesString( String sTableName );

    /**
     * Builds and returns the SQL query string used to drop the given table.
     *
     * @param sTableName  the name of the table to be dropped
     * @return            the fully qualified SQL string
     */
    public abstract String getDropTableString( String sTableName );

    /**
     * Builds and returns the SQL query string used to rename the given table.
     *
     * @param sOldTableName  the name of the table to be renamed
     * @param sNewTableName  the name of the table after the renaming
     * @return               the fully qualified SQL string
     */
    public abstract String getRenameTableString( String sOldTableName, String sNewTableName );

    /**
     * Builds and returns the SQL update string which updates the (unique) node with
     * the given id with the given update/reindexing time. This method is used to
     * update the parent node of a reindexed section which itself is not updated
     * during reindexing time.
     *
     * @param sTableName   the name of the table to be dropped
     * @param sNodeID      the id of the (unique) node to be updated with the given
     *      time
     * @param lUpdateTime  the time to set to the "lastindexed" field
     * @return             the fully qualified SQL string
     */
    public abstract String getUpdateNodeIDString( String sTableName, String sNodeID, long lUpdateTime );

    /**
     * Builds and returns the (sub)string of the final SQL string that is responsible
     * for result limitation on the database query's amount of results.
     *
     * @return   the result-count-limitating SQL string (part)
     */
    public abstract String getLimitationString();

    /**
     * Builds and returns the SQL string which is needed to get the count of the
     * <i>child files</i> (no directories!) of the given node.
     *
     * @param sNodeID  the id of the (unique) node to get the children of
     * @return         the SQL string which runs the query that returns a single
     *      result column containing the amount of children of the given node
     */
    public abstract String getChildCountString( String sNodeID );

    /**
     * Sets the internal lower-limit value to iLowerLimit or 0, if lower than 0.
     * The lower-limit value is used to avoid the loading of too many FileListerEntry
     * instances and must be supported by / implemented in the extending classes.
     *
     * @param iLowerLimit  the new lower limit, but at least 0
     */
    protected void setLowerLimit( int iLowerLimit )
    {
        this.iLowerLimit = ( iLowerLimit >= 0 ) ? iLowerLimit : 0;
    }

    /**
     * Sets the internal upper-limit value to iUpperLimit or 0, if lower than 0.
     * The upper-limit value is used to avoid the loading of too many FileListerEntry
     * instances and must be supported by / implemented in the extending classes.
     *
     * @param iUpperLimit  the new upper limit, but at least 0
     */
    protected void setUpperLimit( int iUpperLimit )
    {
        this.iUpperLimit = ( iUpperLimit >= 0 ) ? iUpperLimit : 0;
    }

    /**
     * Returns the insert SQL string for a PreparedStatement class used by FileLister's
     * Dumpers.
     *
     * @param sTableName  the name of the table to insert entries into
     * @return            the fully qualified SQL string for <i>a PreparedStatement
     *      </i>
     */
    protected final String getInsertString( String sTableName )
    {
        return new StringBuffer( "insert into " ).append( sTableName )
                .append( " (name, directory, size, lastmodified, type, lastindexed, node_id)" )
                .append( " values (?, ?, ?, ?, ?, ?, ?)" ).toString();
    }
}
