package org.alltimeflashdreamer.filelister;

import java.io.*;
import java.net.SocketException;
import java.util.ArrayList;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import org.alltimeflashdreamer.util.StringUtils;

/**
 * <p>
 *
 * Title: ZipBuilderThread</p> <p>
 *
 * Description: Starts and controls the generation of a zip file using the ZipBuilder
 * class. While the zip builder runs it's job, this thread collects the created output
 * and provides it to the FileListerDownloadServlet which then shows a comment site
 * to the client.</p> <p>
 *
 * Organization: All Time Flash Dreamer</p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */

class ZipBuilderThread extends Thread
{
    private String sZipFileName;
    private File[] zipFiles;
    private FileListerConfig fileListerConfig;
    private FileListerZipBuilder zipBuilder;
    private boolean bIsReadyToStream = false;
    private boolean bHundredPercentPageWasShown = false;
    private long lSizeOfFilesToZip = 0L;
    private int iTotalFileCountToZip = 0;
    private long lStartingTime;
    private boolean bIsMarkedForCleanUp = false;

    /**
     * The constructor stores the given pointers and generates a random-based zip
     * file name.
     *
     * @param fileListerConfig        the magic config object
     * @param arrayFileListerEntries  list of selected FileListerEntry objects
     * @throws FileListerException    if the list of entries can't be transformed
     *      into a list of File objects
     */
    ZipBuilderThread( FileListerConfig fileListerConfig, ArrayList arrayFileListerEntries )
        throws FileListerException
    {
        lStartingTime = System.currentTimeMillis();
        sZipFileName = fileListerConfig.getZipCreationDirectory() + FileListerZipBuilder.getFileName( null );
        this.zipFiles = getZipFiles( arrayFileListerEntries, fileListerConfig );
        this.fileListerConfig = fileListerConfig;
        BufferedOutputStream bos = null;
        try
        {
            bos = new BufferedOutputStream( new FileOutputStream( sZipFileName ) );
            zipBuilder = new FileListerZipBuilder( fileListerConfig, bos );
            //DEBUG: System.out.println( "ZipBuilder created. Is null? " + ( zipBuilder == null ) );
        } catch ( Exception ex )
        {
            //store exception to be shown later
            fileListerConfig.setFileListerException( ( ex instanceof FileListerException )
                    ? ( FileListerException )ex : new FileListerException( 41, ex ) );
            try
            {
                bos.close();
            } catch ( Exception ex2 )
            {
            }
        }
    }

    /**
     * Opens a buffered file output stream to the file having the previously generated
     * zip file name and calls the ZipBuilder constructor using this output stream
     * as well as the file list and the config object received in the constructor.
     * The thread is running/alive as long as the ZipBuilder hasn't finished. If
     * an exception occurs, it's wrapped into a FileListerException and set to the
     * FileListerConfig.
     *
     * @see   ZipBuilderThread#getZipFileName
     */
    public void run()
    {
        try
        {
            //build a zip into the previously created output stream (output stream is then closed)
            zipBuilder.buildZip( zipFiles, fileListerConfig );
            if ( !bIsMarkedForCleanUp )
                bIsReadyToStream = true;
            else
                deleteZipFile();
        } catch ( Exception ex )
        {
            //store exception to be shown later
            fileListerConfig.setFileListerException( ( ex instanceof FileListerException )
                    ? ( FileListerException )ex : new FileListerException( 41, ex ) );
            try
            {
                zipBuilder.closeOutputStreams();
            } catch ( Exception ex2 )
            {
            }
            fileListerConfig.setZipBuilderThread( null );
            deleteZipFile();
        }
    }

    /**
     * Override of Object's finalize method: Makes sure that the generated zip is
     * deleted before this class is finalized.
     *
     * @throws Throwable  as defined in Object, but not used
     */
    protected void finalize()
        throws Throwable
    {
        deleteZipFile();
    }

    /**
     * Returns the list of the collected comments of the ZipBuilder while zipping
     * directories and adding file to the zip. If the ZipBuilder couldn't be instantiated,
     * an empty list is returned, but not null.
     *
     * @return   a list of String comments
     */
    ArrayList getComments()
    {
        try
        {
            return zipBuilder.getComments();
        } catch ( Exception ex )
        {
            return new ArrayList();
        }
    }

    /**
     * Returns the name of the generated zip file for having access to it before
     * streaming it to the client.
     *
     * @return   the zip file name generated during construction
     */
    String getZipFileName()
    {
        return sZipFileName;
    }

    /**
     * Opens a buffered file input stream and connects it to the given response's
     * output stream. After the file has been streamed (or some exceptionoccurred),
     * the method closes the input stream, deletes the streamedfile and marks the
     * class as !isReadyToStream, indicating that this file can't be streamed anymore.
     *
     * @param response      where to get the output stream from
     * @throws IOException  <i>only</i> if the redirect to the error site throws
     *      an IOException, i.e. within the method's own catch block
     */
    void streamZipFile( HttpServletResponse response )
        throws IOException
    {
        BufferedInputStream bis = null;
        try
        {
            if ( !isReadyToStream() )
                throw new FileListerException( 46 );

            //open a buffered file input stream and connect it to the given output stream
            response.setContentType( "application/zip" );
            response.setHeader( "Content-Disposition", "attachment; filename=filelister_download.zip;" );
            ServletOutputStream out = response.getOutputStream();
            bis = new BufferedInputStream( new FileInputStream( getZipFileName() ) );
            byte[] data = new byte[1500];
            int iByteCount = 0;
            while ( ( iByteCount = bis.read( data, 0, data.length ) ) > 0 )
            {
                out.write( data, 0, iByteCount );
                out.flush();
            }
        } catch ( Exception ex )
        {
            //Problem 1: In case an exception occurs because the user pressed "cancel" while downloading
            //the zip file, the response is already committed
            //Problem 2: Client stops the browser, the "connection is reset by peer"
            // -> just don't react and close all streams, i.e. react in all other cases only
            if ( !( ( ex instanceof IOException ) && ex.getMessage().equalsIgnoreCase( "Broken pipe" ) ) &&
                    !( ex instanceof SocketException ) )
            {
                //store exception to be shown later
                if ( ex instanceof FileListerException )
                    fileListerConfig.setFileListerException( ( FileListerException )ex );
                else
                    fileListerConfig.setFileListerException( new FileListerException( 41, ex ) );

                //avoid producing an IllegalStateException...
                if ( !response.isCommitted() )
                    response.sendRedirect( fileListerConfig.getErrorURL() );
            }
        }

        //close all the streams
        try
        {
            bis.close();
        } catch ( Exception ex )
        {
        }

        //delete the temp file, mark as "no more ready to stream" and unset the
        //FileListerConfig's pointer
        deleteZipFile();
        fileListerConfig.setZipBuilderThread( null );
        bIsReadyToStream = false;

        //flush and close the output stream, if still open, to notify the client's
        //browser that the session is finished anyway
        try
        {
            response.getOutputStream().flush();
            response.getOutputStream().close();
        } catch ( Exception ex )
        {
        }
    }

    /**
     * Returns whether the run method successfully completed the zip file.
     *
     * @return   zip file streamable?
     */
    boolean isReadyToStream()
    {
        return bIsReadyToStream;
    }

    /**
     * Returns the result of the division of the size of the processed files and
     * the total size of files to be processed.
     *
     * @return   e.g. 0.18, indicating that 18% of the total byte count is already
     *      packed into the archive
     */
    double getProgressPercentage()
    {
        try
        {
            if ( lSizeOfFilesToZip == 0L )
                return 0.0d;
            double dCalculatedPercentage = ( double )zipBuilder.getSizeOfProcessedFiles() / ( double )lSizeOfFilesToZip;
            return ( ( dCalculatedPercentage > 1.0 ) ? 1.0 : dCalculatedPercentage );
        } catch ( NullPointerException npe )
        {
            //DEBUG: System.out.println( "ZipBuilderThread.getProgressPercentage: Caught NullPointerException!" );
            return 0.0d;
        }
    }

    /**
     * Simple accessor method for ZipBuilder's getCurrentArchiveSize() method.
     *
     * @return   same as getCurrentArchiveSize() in ZipBuilder class
     * @see      org.alltimeflashdreamer.util.zip.ZipBuilder#getCurrentArchiveSize
     */
    long getCurrentArchiveSize()
    {
        try
        {
            return zipBuilder.getCurrentArchiveSize();
        } catch ( NullPointerException npe )
        {
            //DEBUG: System.out.println( "ZipBuilderThread.getCurrentArchiveSize: Caught NullPointerException!" );
            return 0L;
        }
    }

    /**
     * Simple accessor method for ZipBuilder's getOverallPackingRatio() method.
     *
     * @return   same as getOverallPackingRatio() in ZipBuilder class
     * @see      org.alltimeflashdreamer.util.zip.ZipBuilder#getOverallPackingRatio
     */
    double getOverallPackingRatio()
    {
        try
        {
            return zipBuilder.getOverallPackingRatio();
        } catch ( NullPointerException npe )
        {
            //DEBUG: System.out.println( "ZipBuilderThread.getOverallPackingRatio: Caught NullPointerException!" );
            return 0.0d;
        }
    }

    /**
     * Returns the amount of processed files and the total amount of files to be
     * added to the zip.
     *
     * @return   something like "29 / 314"
     */
    String getFilesDone()
    {
        try
        {
            int iProcessedFilesCount = zipBuilder.getProcessedFilesCount();
            return new StringBuffer().append( iProcessedFilesCount ).append( " / " )
                    .append( ( iTotalFileCountToZip >= iProcessedFilesCount ) ? iTotalFileCountToZip : iProcessedFilesCount )
                    .toString();
        } catch ( NullPointerException npe )
        {
            //DEBUG: System.out.println( "ZipBuilderThread.getFilesDone: Caught NullPointerException!" );
            return "0 / " + iTotalFileCountToZip;
        }
    }

    /**
     * Returns the amount of bytes already processed and the total amount of bytes
     * to be processed.
     *
     * @return   something like "2.34 MB / 9.789 MB"
     */
    String getSizeDone()
    {
        long lSizeOfProcessedFiles = ( zipBuilder == null ) ? 0L : zipBuilder.getSizeOfProcessedFiles();
        return new StringBuffer( StringUtils.formatSize( lSizeOfProcessedFiles ) ).append( " / " )
                .append( StringUtils.formatSize( lSizeOfFilesToZip ) ).toString();
    }

    /**
     * Simple accessor method for ZipBuilder's getCurrentlyAddedFile() method.
     *
     * @return   same as getCurrentlyAddedFile() in ZipBuilder class
     * @see      org.alltimeflashdreamer.util.zip.ZipBuilder#getCurrentlyAddedFile
     */
    String getCurrentlyAdding()
    {
        try
        {
            return zipBuilder.getCurrentlyAddedFile();
        } catch ( NullPointerException npe )
        {
            //DEBUG: System.out.println( "ZipBuilderThread.getCurrentlyAddedFile: Caught NullPointerException!" );
            return "";
        }
    }

    /**
     * Returns the amount of elapsed time and the expected amount or remaining time.
     * There are two special cases:
     * <ul>
     *   <li> 0% completed: a question mark is shown indicating that the program
     *   has no idea about how long it will take
     *   <li> 100% completed: there's no remaining time at all
     * </ul>
     *
     *
     * @return   something like "1 min 27 sec (less than 1 minute remaining)"
     */
    String getTimeElapsed()
    {
        long lElapsedMilliseconds = System.currentTimeMillis() - lStartingTime;
        StringBuffer sb = new StringBuffer( "time elapsed: " ).append( StringUtils.formatPeriod( lElapsedMilliseconds ) );
        try
        {
            double dProgressPercentage = getProgressPercentage();
            long lRemainingMilliseconds = ( long )( lElapsedMilliseconds / dProgressPercentage );
            if ( dProgressPercentage != 1.0 )
                sb.append( " (about " ).append( ( dProgressPercentage == 0.0 ) ? "?" :
                        StringUtils.formatPeriod( lRemainingMilliseconds ) ).append( " remaining)" );
        } catch ( NullPointerException npe )
        {
            //DEBUG: System.out.println( "ZipBuilderThread.getTimeElapsed: Caught NullPointerException!" );
        }
        ;

        return sb.toString();
    }

    ArrayList getWarnings()
    {
        /**
         * @todo   Implement this functionality
         */
        return new ArrayList();
    }

    /**
     * Returns whether the final / 100% page has already been shown or not.
     *
     * @return   last page was shown?
     */
    boolean hasHundredPercentPageBeenShown()
    {
        return bHundredPercentPageWasShown;
    }

    /** Tells that the final / 100% page has now been shown. */
    void setHundredPercentPageWasShown()
    {
        this.bHundredPercentPageWasShown = true;
    }

    /**
     * Sets the "mark for clean-up" boolean. This leads to an invalidation of the
     * whole thread, i.e. it's never ready to be streamed, and the zip file is deleted
     * immediately after the creation.
     */
    void markForCleanUp()
    {
        bIsMarkedForCleanUp = true;
    }

//-------------------------------------- private methods ---------------------------------//
    /**
     * Tries to delete the generated zip file out of the zip generation directory.
     */
    private void deleteZipFile()
    {
        try
        {
            new File( getZipFileName() ).delete();
        } catch ( Exception ex )
        {
        }
    }

    /**
     * Builds the list of files and directories to be added to the zip file. In case
     * a directory contains subdirectories, they are added to the list too, recursively.
     * While stepping through the list, the sum of the sizes of the FileListerEntry
     * objects is built.
     *
     * @param arrayFileListerEntries  list of selected FileListerEntry objects
     * @param fileListerConfig        the magic config object
     * @return                        the list of File objects to be packed into
     *      the zip file
     * @throws FileListerException    each exception that may occur is packed and
     *      described with a separate FileListerException
     */
    private File[] getZipFiles( ArrayList arrayFileListerEntries, FileListerConfig fileListerConfig )
        throws FileListerException
    {
        FileListerEntry nextFle;
        File nextFileOrDir;
        File[] filesAndDirectories = new File[arrayFileListerEntries.size()];
        for ( int i = 0; i < arrayFileListerEntries.size(); i++ )
        {
            nextFle = ( FileListerEntry )arrayFileListerEntries.get( i );
            lSizeOfFilesToZip += nextFle.getSize();
            nextFileOrDir = new File( nextFle.getDirectory(), nextFle.getName() );
            if ( !nextFileOrDir.exists() )
                throw new FileListerException( 36, new FileNotFoundException( "Selected entry \"" + nextFileOrDir.getName() + "\" is missing!" ) );
            else if ( !nextFileOrDir.canRead() )
                throw new FileListerException( 36, new FileNotFoundException( "Selected entry \"" + nextFileOrDir.getName() + "\" can't be read!" ) );
            else
            {
                filesAndDirectories[i] = nextFileOrDir;
                if ( filesAndDirectories[i].isFile() )
                    iTotalFileCountToZip++;
                else
                    iTotalFileCountToZip += fileListerConfig.getFileLister().getChildCount( nextFle );
            }
        }

        return filesAndDirectories;
    }
}
