package org.alltimeflashdreamer.filelister.gui;

import java.awt.Component;
import java.awt.FontMetrics;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableCellRenderer;

/**
 * <p>
 *
 * Title: FileListerCellRenderer</p> <p>
 *
 * Description: This cell renderer is designed to render the property value column
 * of the SetupFrame.</p> <p>
 *
 * Organization: All Time Flash Dreamer</p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */

class FileListerCellRenderer implements TableCellRenderer
{
    private FileListerTableModel model;
    private EmptyBorder emptyBorder;
    private JComponent[][] components;

    /**
     * Constructor uses the given model pointer to build up the array of components
     * representing the table cell entries.
     *
     * @param model  the table model pointer
     */
    FileListerCellRenderer( FileListerTableModel model )
    {
        this.model = model;
        components = new JComponent[model.getRowCount()][model.getColumnCount()];
        for ( int i = 0; i < components.length; i++ )
            components[i][0] = new MyTitleJLabel( model.getValueAt( i, 0 ) );
        updateValueComponents();
        emptyBorder = new EmptyBorder( 0, 0, 0, 0 );
    }

    /**
     * Returns the rendering component of the cell at the given coordinates within
     * the table.
     *
     * @param table        the JTable to be rendered
     * @param value        the cell value
     * @param bIsSelected  whether the cell is selected or not
     * @param bHasFocus    whether the cell has the focus or not
     * @param iRow         which row is to be rendered
     * @param iColumn      which column is to be rendered
     * @return             a Component to be filled into the cell
     */
    public Component getTableCellRendererComponent( JTable table, Object value,
            boolean bIsSelected, boolean bHasFocus, int iRow, int iColumn )
    {
        return getComponentAt( iRow, iColumn );
    }

    /**
     * Returns the component at the given position out of the internal array of stored
     * components.
     *
     * @param iRow     the row of the component to return
     * @param iColumn  the column of the component to return
     * @return         the component at the given position (if the position is out
     *      of range, an ArrayIndexOutOfBoundsException is thrown)
     */
    JComponent getComponentAt( int iRow, int iColumn )
    {
        return components[iRow][iColumn];
    }

    /**
     * Updates/Reinitializes the list of <i>value</i> components (only those that
     * are editable).
     */
    void updateValueComponents()
    {
        for ( int i = 0; i < components.length; i++ )
        {
            Object oValue = model.getValueAt( i, 1 );
            if ( oValue instanceof Boolean )
                components[i][1] = new MyJCheckBox( oValue );
            else if ( oValue instanceof Integer )
                components[i][1] = new MyIntegerJLabel( oValue );
            else
                components[i][1] = new MyStringJLabel( oValue );
        }
    }

    /**
     * Returns the size (in pixels) of the longest property name.
     *
     * @return   the maximum string width of the properties
     */
    int getMaximumNameSize()
    {
        int iMaximumNameSize = 0;
        FontMetrics fm = ( ( MyTitleJLabel )components[0][0] ).getFontMetrics( ( ( MyTitleJLabel )components[0][0] ).getFont() );
        for ( int i = 0; i < components.length; i++ )
        {
            int iNextStringWidth = fm.stringWidth( ( ( MyTitleJLabel )components[i][0] ).getText() );
            if ( iNextStringWidth > iMaximumNameSize )
                iMaximumNameSize = iNextStringWidth;
        }

        return iMaximumNameSize;
    }

//---------------------- inner class MyJCheckBox -------------------------------//
    private class MyJCheckBox extends JCheckBox
    {
        private MyJCheckBox( Object booleanObject )
        {
            super( "", ( ( Boolean )booleanObject ).booleanValue() );
            setHorizontalAlignment( SwingConstants.CENTER );
            setBackground( SetupFrame.BGCOLOR_1 );
        }
    }

//---------------------- inner class MyTitleJLabel -------------------------------//
    private class MyTitleJLabel extends JLabel
    {
        private MyTitleJLabel( Object stringValue )
        {
            super( ( String )stringValue );
            setBackground( SetupFrame.BGCOLOR_1 );
            setFont( SetupFrame.FONT_BOLD );
        }
    }

//---------------------- inner class MyStringJLabel -------------------------------//
    private class MyStringJLabel extends JLabel
    {
        private MyStringJLabel( Object stringValue )
        {
            super( ( String )stringValue );
            setBackground( SetupFrame.BGCOLOR_1 );
            setBorder( emptyBorder );
            setFont( SetupFrame.FONT_PLAIN );
        }
    }

//---------------------- inner class MyIntegerJLabel -------------------------------//
    private class MyIntegerJLabel extends JLabel
    {
        private MyIntegerJLabel( Object integerValue )
        {
            super( ( ( Integer )integerValue ).toString() );
            setBackground( SetupFrame.BGCOLOR_1 );
            setBorder( emptyBorder );
            setFont( SetupFrame.FONT_PLAIN );
        }
    }
}
