package org.alltimeflashdreamer.filelister.gui;

import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ChangeEvent;

/**
 * <p>
 *
 * Title: FileListerTable</p> <p>
 *
 * Description: This class represents the table containing the property names and
 * values. While the property name column isn't editable at all, the property values
 * are editable and rendered according to the individual value classes.</p> <p>
 *
 * Organization: All Time Flash Dreamer</p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */

public class FileListerTable extends JTable
{
    private boolean bIsEdited = false;
    private FileListerTextArea jtDescriptions;

    /**
     * Constructor receives the table model and configures the table with the data
     * and the selection model, the cell renderer and the cell editor.
     *
     * @param model           the (data) model containing the config properties
     * @param jtDescriptions  the text area containing the property descriptions
     */
    public FileListerTable( FileListerTableModel model, FileListerTextArea jtDescriptions )
    {
        setModel( model );
        setBackground( SetupFrame.BGCOLOR_1 );
        this.jtDescriptions = jtDescriptions;
        this.jtDescriptions.setDescriptions( model.getXMLHandler().getDescriptions() );
        getColumnModel().getSelectionModel().setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        getColumnModel().setColumnSelectionAllowed( false );
        tableHeader.setReorderingAllowed( false );
        tableHeader.setFont( SetupFrame.FONT_BOLD );
        /**
         * @todo   WHAT'S WRONG with "getColumnModel().getColumn( 0 ).setPreferredWidth(
         *      renderer.getMaximumNameSize() + 5 );" ?
         */

        //add sophisticated mouse listener for the handling of the checkboxes...
        addMouseListener(
            new MouseAdapter()
            {
                /**
                 * Calculates which component was clicked. If the clicked component
                 * was a checkbox, the method programmatically fakes a click on the
                 * checkbox and updates both the component and the model.
                 *
                 * @param event  the event to handle (only executed if one left button
                 *      click)
                 */
                public void mouseClicked( MouseEvent event )
                {
                    int iRow = getRowAtPoint( event.getPoint() );
                    int iColumn = getColumnAtPoint( event.getPoint() );
                    JComponent comp = ( ( FileListerCellRenderer )getCellRenderer( iRow, iColumn ) ).getComponentAt( iRow, iColumn );
                    if ( comp instanceof JCheckBox && event.getClickCount() == 1 && event.getModifiers() == MouseEvent.BUTTON1_MASK )
                    {
                        JCheckBox checkbox = ( JCheckBox )comp;
                        checkbox.setSelected( !checkbox.isSelected() );
                        getModel().setValueAt( new Boolean( checkbox.isSelected() ), iRow, iColumn );
                        setEdited( true );
                        event.consume();
                    }
                }
            } );
        addMouseMotionListener(
            new MouseMotionAdapter()
            {
                /**
                 * Updates the description text window.
                 *
                 * @param event  the mouse motion event to be handled
                 */
                public void mouseMoved( MouseEvent event )
                {
                    FileListerTable.this.jtDescriptions.setDescription( getRowAtPoint( event.getPoint() ) );
                }
            } );

        reloadRenderer();
        setCellEditor( new FileListerCellEditor() );
    }

    /**
     * Override of superclass method, calls the FileListerCellRenderer's updateValueComponents
     * method afterwards.
     *
     * @param event  the ChangeEvent which lead to the stopping of the editing
     */
    public void editingStopped( ChangeEvent event )
    {
        super.editingStopped( event );
        ( ( FileListerCellRenderer )getCellRenderer( 0, 1 ) ).updateValueComponents();
        setEdited( true );
    }

    /**
     * Returns whether the table was edited or not. This value is set to true after
     * the "successful" editing of a call, regardless of whether the cell's content
     * has been changed or not.
     *
     * @return   has the model of the table been changed?
     */
    public boolean isEdited()
    {
        return bIsEdited;
    }

    /**
     * Sets the isEdited parameter.
     *
     * @param bIsEdited  set/unset the "is edited" state of the table
     */
    public void setEdited( boolean bIsEdited )
    {
        this.bIsEdited = bIsEdited;
    }

    /** (Re)Loads the cell renderer and (re)sets it to all the columns. */
    public void reloadRenderer()
    {
        FileListerCellRenderer renderer = new FileListerCellRenderer( ( FileListerTableModel )getModel() );
        for ( int i = 0; i < getColumnCount(); i++ )
            getColumn( getColumnName( i ) ).setCellRenderer( renderer );
    }

//------------------------------------ inner methods -------------------------------------//
    private int getRowAtPoint( Point point )
    {
        int iRelY = point.y - getY();
        if ( iRelY < 0 || iRelY > getHeight() )
            return -1;

        return ( iRelY / getRowHeight() );
    }

    private int getColumnAtPoint( Point point )
    {
        return getColumnModel().getColumnIndexAtX( point.x );
    }
}
