package org.alltimeflashdreamer.filelister.gui;

import java.io.File;
import java.io.IOException;

import javax.swing.table.DefaultTableModel;

import org.alltimeflashdreamer.filelister.xml.FileListerXMLHandler;
import org.xml.sax.SAXException;

/**
 * <p>
 *
 * Title: FileListerTableModel</p> <p>
 *
 * Description: This is the table model which shows the list of properties needed
 * by the FileLister application.</p> <p>
 *
 * Organization: All Time Flash Dreamer</p> <p>
 *
 * </p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */

public class FileListerTableModel extends DefaultTableModel
{
    private final static String[] COLUMNNAMES = {"Property names", "Property values"};

    private FileListerXMLHandler xmlHandler;

    /**
     * Constructor initializes the (default) parameters of the property file (unverified)
     * and stores them within the table model.
     *
     * @param propertyFile   the property file
     * @throws IOException   if the property file wasn't found
     * @throws SAXException  if the property file couldn't be parsed
     */
    public FileListerTableModel( File propertyFile )
        throws IOException, SAXException
    {
        load( propertyFile, false );
    }

    /**
     * Returns the column count of the table.
     *
     * @return   how many columns are there in the table?
     */
    public int getColumnCount()
    {
        return COLUMNNAMES.length;
    }

    /**
     * Returns the name of the column having the given number.
     *
     * @param iColumn  the number of the column
     * @return         the name of the given column
     */
    public String getColumnName( int iColumn )
    {
        return COLUMNNAMES[iColumn];
    }

    /**
     * Returns whether the given cell is editable. This is only true if the given
     * cell is a member of the <i>value column</i> .
     *
     * @param iRow     the number of the row
     * @param iColumn  the number of the column
     * @return         only true if the second column
     */
    public boolean isCellEditable( int iRow, int iColumn )
    {
        if ( iColumn == 0 )
            return false;

        return ( !( this.getValueAt( iRow, iColumn ) instanceof Boolean ) );
    }

    /**
     * Returns the config reader, e.g. for save actions.
     *
     * @return   the config reader class
     */
    public FileListerXMLHandler getXMLHandler()
    {
        return xmlHandler;
    }

    /**
     * Clears the whole table, reloads the configuration and notifies the table about
     * the change in data.
     *
     * @param propertyFile   the file to (re)load
     * @param bLoadDefaults  whether to load the default values or not
     * @throws IOException   if the property file wasn't found
     * @throws SAXException  if the property file couldn't be parsed
     */
    public void reload( File propertyFile, boolean bLoadDefaults )
        throws IOException, SAXException
    {
        for ( int i = 0; i < this.getRowCount(); i++ )
            this.removeRow( 0 );
        load( propertyFile, bLoadDefaults );
        fireTableDataChanged();
    }

//---------------------------- inner methods -------------------------------------//
    private void load( File propertyFile, boolean bLoadDefaults )
        throws IOException, SAXException
    {
        //read the config file
        xmlHandler = new FileListerXMLHandler( propertyFile, bLoadDefaults );

        //set the data object of the TableModel
        setDataVector( xmlHandler.getData(), COLUMNNAMES );
    }
}
