package org.alltimeflashdreamer.filelister.xml;

import java.util.ArrayList;

import org.alltimeflashdreamer.filelister.gui.FileListerTableModel;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * <p>
 *
 * Title: AbstractConfigElementList</p> <p>
 *
 * Description: This is the superclass of each list of ConfigElement classes, representing
 * all together one "main node" of the FileLister config file.</p> <p>
 *
 * Organization: All Time Flash Dreamer</p> <p>
 *
 * </p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */

abstract class AbstractConfigElementList extends ArrayList
{
    private String sName;

    /**
     * Initializes the list with the given Element node. This leads to a scanning
     * of the node for children of type Element; for each such child, an instance
     * of ConfigElement is added to the list.
     *
     * @param element        the Element representing the node containing all the
     *      child elements / config settings of this node
     * @param bLoadDefaults  whether to load the default values or not
     */
    AbstractConfigElementList( Element element, boolean bLoadDefaults )
    {
        this.sName = element.getNodeName();
        Element childElement;
        for ( Node child = element.getFirstChild(); child != null; child = child.getNextSibling() )
        {
            try
            {
                childElement = ( Element )child;
                this.add( new ConfigElement( childElement, bLoadDefaults ) );
            } catch ( ClassCastException cce )
            {
            }
        }
    }

    /**
     * Returns the name of the ConfigElement at the given position. If this position
     * is out of range, null is returned. The name is formatted like "&lt;list-name&gt;.&lt;element-name&gt;"
     *
     * @param iPosition  the position of the wished element within the list
     * @return           the name of the ConfigElement at the given position or null
     */
    public String getPropertyName( int iPosition )
    {
        if ( iPosition < 0 || iPosition >= size() )
            return null;
        return new StringBuffer( sName ).append( "." ).append( ( ( ConfigElement )get( iPosition ) ).getName() ).toString();
    }

    /**
     * Returns the value of the ConfigElement at the given position. If this position
     * is out of range, null is returned.
     *
     * @param iPosition  the position of the wished element within the list
     * @return           the value of the ConfigElement at the given position or
     *      null
     */
    public Object getPropertyValue( int iPosition )
    {
        if ( iPosition < 0 || iPosition >= size() )
            return null;
        return ( ( ConfigElement )get( iPosition ) ).getValue();
    }

    /**
     * Returns the value of the ConfigElement having the given name <i>as a String
     * </i>. If this name isn't known, null is returned.
     *
     * @param sName  the name of the wished element within the list
     * @return       the value of the ConfigElement having the given name or null
     */
    public String getPropertyValue( String sName )
    {
        ConfigElement nextElement;
        for ( int i = 0; i < size(); i++ )
        {
            nextElement = ( ConfigElement )get( i );
            if ( nextElement.getName().equals( sName ) )
                return nextElement.getValueString();
        }
        return null;
    }

    /**
     * Returns the name of the list's underlying element.
     *
     * @return   the list name
     */
    public String getName()
    {
        return sName;
    }

    /**
     * Returns the element with the given name. If this element isn't found in the
     * list, null is returned.
     *
     * @param sElementName  a name specifying an ConfigElement instance
     * @return              the specified element or null
     */
    public ConfigElement getConfigElement( String sElementName )
    {
        ConfigElement nextElement;
        for ( int i = 0; i < size(); i++ )
        {
            nextElement = ( ConfigElement )get( i );
            if ( nextElement.getName().equals( sElementName ) )
                return nextElement;
        }
        return null;
    }

    /**
     * Searches the model for the properties named "getName().&lt;some-name&gt;".
     * The valueo of each such property is stored within the related ConfigElement,
     * i.e. within the DOM itself.
     *
     * @param model  the model to get the property names and values from
     */
    public void setValues( FileListerTableModel model )
    {
        for ( int i = 0; i < model.getRowCount(); i++ )
        {
            String sPropertyName = ( String )model.getValueAt( i, 0 );
            if ( sPropertyName.startsWith( getName() ) )
            {
                sPropertyName = sPropertyName.substring( getName().length() + 1 );
                ConfigElement nextElement;
                for ( int j = 0; j < size(); j++ )
                {
                    nextElement = ( ConfigElement )get( j );
                    if ( nextElement.getName().equals( sPropertyName ) )
                        nextElement.setValue( model.getValueAt( i, 1 ) );
                }
            }
        }
    }
}
