package org.alltimeflashdreamer.filelister.xml;

import org.w3c.dom.*;

/**
 * <p>
 *
 * Title: ConfigElement</p> <p>
 *
 * Description: This class represents one element node from the config file.</p>
 * <p>
 *
 * Organization: All Time Flash Dreamer</p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */

public class ConfigElement
{
    /** stores the pointer to the underlying Element node */
    private Element element;
    /** stores the name of the element */
    private String sName;
    /** stores the default value of the element */
    private String sDefault;
    /** stores the effective/active value of the element */
    private Object oValue;
    /** stores the description of the element */
    private String sDescription;
    /** stores the class of the element */
    private Class clazz;

    /**
     * Initializes the class with the given Element node. This leads to a scanning
     * of the node for its name as well as the "default" and the "value" attribute
     * values. Additionally, the text content of the node is set as the description.
     *
     * @param element        the Element representing the node containing all the
     *      attributes and the description of this node
     * @param bLoadDefaults  whether to load the default values or not
     */
    ConfigElement( Element element, boolean bLoadDefaults )
    {
        this.element = element;
        sName = element.getNodeName();
        sDefault = element.getAttribute( "default" );
        try
        {
            this.sDescription = findFirstText( element ).getData();
        } catch ( NullPointerException npe )
        {
            this.sDescription = "";
        }
        try
        {
            this.clazz = Class.forName( element.getAttribute( "class" ) );
        } catch ( Exception ex )
        {
            this.clazz = String.class;
        }

        //set the appropriate object (type)
        setValue( ( bLoadDefaults ) ? sDefault : element.getAttribute( "value" ) );
    }

    /**
     * Returns the name of the element.
     *
     * @return   the element name
     */
    public String getName()
    {
        return sName;
    }

    /**
     * Returns the value of the element. This is either an Integer, a Boolean or
     * a String.
     *
     * @return   the value object of the element's "value" attribute node
     */
    public Object getValue()
    {
        return oValue;
    }

    /**
     * Returns the value of the element <i>as a String</i> .
     *
     * @return   the value of the element's "value" attribute node, formatted as
     *      a String
     */
    public String getValueString()
    {
        return toString( oValue );
    }

    /**
     * Sets/Updates the value attribute of the element.
     *
     * @param oValue  the new value object to be set to the value attribute
     */
    public void setValue( Object oValue )
    {
        element.setAttribute( "value", toString( oValue ) );
        setValue( element.getAttribute( "value" ) );
    }

    /**
     * Returns the default (value) of the element.
     *
     * @return   the value of the element's "default" attribute node
     */
    public String getDefault()
    {
        return sDefault;
    }

    /**
     * Returns the description of the element.
     *
     * @return   the text value of the element's content
     */
    public String getDescription()
    {
        return sDescription;
    }

    /**
     * Returns the class type of the element. Because Object's getClass method is
     * final and can't be overridden, this method is named getClazz...
     *
     * @return   the value of the element's "class" attribute node, transformed into
     *      a class
     */
    public Class getClazz()
    {
        return clazz;
    }

//-------------------------------- inner methods ---------------------------------------//
    /**
     * Finds the first text node of an descendent element and returns it. This recursively
     * searches more than one level and looks for text in font nodes, etc.
     *
     * @param node  The starting node for the search.
     * @return      The text node or null if not found.
     */
    private Text findFirstText( Node node )
    {
        if ( node instanceof Text )
            return ( Text )node;

        for ( Node child = node.getFirstChild(); child != null; child = child.getNextSibling() )
        {
            Text text = findFirstText( child );
            if ( text != null )
                return text;
        }
        return null;
    }

    /**
     * Formats the given object into a String.
     *
     * @param object  the object to format
     * @return        the formatted String
     */
    private String toString( Object object )
    {
        if ( object instanceof Boolean )
            return ( ( Boolean )object ).toString();
        else if ( object instanceof Integer )
            return ( ( Integer )object ).toString();
        else
            return ( String )object;
    }

    /**
     * Sets the value attribute object by converting the given String into the appropriate
     * Object instance (Boolean, Integer or String).
     *
     * @param sValue  the value string to convert into a value object and set to
     *      oValue
     */
    private void setValue( String sValue )
    {
        if ( clazz.getName().equals( Boolean.class.getName() ) )
            this.oValue = new Boolean( sValue );
        else if ( clazz.getName().equals( Integer.class.getName() ) )
            this.oValue = new Integer( sValue );
        else
            this.oValue = new String( sValue );
    }
}
