package org.alltimeflashdreamer.filelister.xml;

import java.io.File;
import java.io.IOException;
import java.util.StringTokenizer;

import org.alltimeflashdreamer.filelister.FileListerException;
import org.alltimeflashdreamer.filelister.PropertyFileParameters;
import org.alltimeflashdreamer.filelister.gui.FileListerTableModel;
import org.alltimeflashdreamer.util.xml.XMLDataHandler;
import org.xml.sax.SAXException;

/**
 * <p>
 *
 * Title: FileListerXMLHandler</p> <p>
 *
 * Description: This is the parser and writer class for reading and writing the FileLister
 * configuration XML file.</p> <p>
 *
 * Organization: All Time Flash Dreamer</p>
 *
 * @author    Copyright (c) 2002-2004 by <a href="mailto:florian@steinsiepe.ch">Florian
 *      Steinsiepe</a>
 * @version   0.5.1
 */

public class FileListerXMLHandler extends XMLDataHandler
{
    /** name of FileLister's configuration file */
    public final static String CONFIGFILE = "filelister.xml";
    /** the jdbc element within the config file */
    private JDBC jdbc;
    /** the directory element within the config file */
    private Directory directory;
    /** the index element within the config file */
    private Index index;
    /** the system element within the config file */
    private org.alltimeflashdreamer.filelister.xml.System system;
    /** the link element within the config file */
    private Link link;
    /** the zip element within the config file */
    private Zip zip;

    /**
     * Calls the superclass' constructor and initializes the internal objects JDBC,
     * directory, index, system, link and zip, each representing one of the major
     * sections in the xml file and being fully configured with the values of its
     * section.
     *
     * @param xmlFile        the file to be parsed
     * @param bLoadDefaults  whether to load the default values or not
     * @throws IOException   thrown on file access
     * @throws SAXException  thrown on parsing of the config file
     */
    public FileListerXMLHandler( File xmlFile, boolean bLoadDefaults )
        throws IOException, SAXException
    {
        super( xmlFile );

        jdbc = new JDBC( getUniqueElement( "jdbc" ), bLoadDefaults );
        directory = new Directory( getUniqueElement( "directory" ), bLoadDefaults );
        index = new Index( getUniqueElement( "index" ), bLoadDefaults );
        system = new org.alltimeflashdreamer.filelister.xml.System( getUniqueElement( "system" ), bLoadDefaults );
        link = new Link( getUniqueElement( "link" ), bLoadDefaults );
        zip = new Zip( getUniqueElement( "zip" ), bLoadDefaults );
    }

    /**
     * Returns a list of key/value pairs, ordered as follows: jdbc, directory, system,
     * index, link, zip. The order within the list elements is defined by the order
     * in the config file.
     *
     * @return   a two-dimensional array of config element names and values
     */
    public Object[][] getData()
    {
        Object[][] oReturnValue = new Object[getElementCount()][2];
        int iCtr = 0;
        for ( int i = 0; i < jdbc.size(); i++ )
        {
            oReturnValue[iCtr][0] = jdbc.getPropertyName( i );
            oReturnValue[iCtr++][1] = jdbc.getPropertyValue( i );
        }
        for ( int i = 0; i < directory.size(); i++ )
        {
            oReturnValue[iCtr][0] = directory.getPropertyName( i );
            oReturnValue[iCtr++][1] = directory.getPropertyValue( i );
        }
        for ( int i = 0; i < system.size(); i++ )
        {
            oReturnValue[iCtr][0] = system.getPropertyName( i );
            oReturnValue[iCtr++][1] = system.getPropertyValue( i );
        }
        for ( int i = 0; i < index.size(); i++ )
        {
            oReturnValue[iCtr][0] = index.getPropertyName( i );
            oReturnValue[iCtr++][1] = index.getPropertyValue( i );
        }
        for ( int i = 0; i < link.size(); i++ )
        {
            oReturnValue[iCtr][0] = link.getPropertyName( i );
            oReturnValue[iCtr++][1] = link.getPropertyValue( i );
        }
        for ( int i = 0; i < zip.size(); i++ )
        {
            oReturnValue[iCtr][0] = zip.getPropertyName( i );
            oReturnValue[iCtr++][1] = zip.getPropertyValue( i );
        }
        return oReturnValue;
    }

    /**
     * Returns a list of description texts (same order as the getData elements).
     *
     * @return   the description list
     */
    public String[] getDescriptions()
    {
        String[] sReturnValue = new String[getElementCount()];
        int iCtr = 0;
        for ( int i = 0; i < jdbc.size(); i++ )
            sReturnValue[iCtr++] = formatDescription( ( ( ConfigElement )jdbc.get( i ) ).getDescription() );
        for ( int i = 0; i < directory.size(); i++ )
            sReturnValue[iCtr++] = formatDescription( ( ( ConfigElement )directory.get( i ) ).getDescription() );
        for ( int i = 0; i < system.size(); i++ )
            sReturnValue[iCtr++] = formatDescription( ( ( ConfigElement )system.get( i ) ).getDescription() );
        for ( int i = 0; i < index.size(); i++ )
            sReturnValue[iCtr++] = formatDescription( ( ( ConfigElement )index.get( i ) ).getDescription() );
        for ( int i = 0; i < link.size(); i++ )
            sReturnValue[iCtr++] = formatDescription( ( ( ConfigElement )link.get( i ) ).getDescription() );
        for ( int i = 0; i < zip.size(); i++ )
            sReturnValue[iCtr++] = formatDescription( ( ( ConfigElement )zip.get( i ) ).getDescription() );
        return sReturnValue;
    }

    /**
     * Reads all the values of the given model and updates the DOM. After this, a
     * new instance of PropertyFileParameters is created to verify the new contents
     * of the DOM. If the check is successful, the whole XML document is written
     * down to the file from which it was parsed. If during this updating of the
     * DOM, the checking of the parameters or the writing of the DOM fails, the upcoming
     * exception is thrown.
     *
     * @param model                 the model containing the modified data
     * @throws IOException          if the writing failed somehow
     * @throws FileListerException  if a change in the configuration isn't allowed
     */
    public void save( FileListerTableModel model )
        throws IOException, FileListerException
    {
        try
        {
            jdbc.setValues( model );
            directory.setValues( model );
            index.setValues( model );
            system.setValues( model );
            link.setValues( model );
            zip.setValues( model );
        } catch ( Throwable t )
        {
            throw new FileListerException( 48, t );
        }

        PropertyFileParameters props = new PropertyFileParameters( this );

        //some values need to be corrected, especially those of the directories; therefore,
        //call the getter methods (those whithout reloading-of-the-xml-file) and set the
        //corrected values back to the DOM tree
        jdbc.getConfigElement( "initialconnectionpoolsize" ).setValue( new Integer( props.getInitialConnections( false ) ) );
        jdbc.getConfigElement( "pooledconnectionincrement" ).setValue( new Integer( props.getConnectionIncrement( false ) ) );
        directory.getConfigElement( "virtualroot" ).setValue( props.getVirtualRootDirectory( false ) );
        String[] sNoGoAreas = ( String[] )props.getNoGoAreas( false ).toArray( new String[]{} );
        StringBuffer sbNoGoAreas = new StringBuffer();
        for ( int i = 0; i < sNoGoAreas.length; i++ )
            sbNoGoAreas.append( sNoGoAreas[i] ).append( ( i < ( sNoGoAreas.length - 1 ) ) ? "," : "" );
        directory.getConfigElement( "nogoarea" ).setValue( sbNoGoAreas.toString() );
        system.getConfigElement( "maxfilelisterobjects" ).setValue( new Integer( props.getMaxFileListerObjects( false ) ) );
        index.getConfigElement( "aging" ).setValue( new Integer( props.getIndexAging( false ) ) );
        zip.getConfigElement( "creationdirectory" ).setValue( props.getZipCreationDirectory( false ) );
        zip.getConfigElement( "packagingstrength" ).setValue( new Integer( props.getZipPackageStrength( false ) ) );

        //if everything is updated, write the DOM tree down to the source file
        write();
    }

    /**
     * Returns the sum of elements of each element list, i.e. the total amount of
     * config settings.
     *
     * @return   the ConfigElement count
     */
    private int getElementCount()
    {
        return jdbc.size() + directory.size() + index.size() + system.size() + link.size() + zip.size();
    }

    /**
     * Formats the description string into a displayable way. This is done be trimming
     * each line.
     *
     * @param sXML  the XML tag's content to format
     * @return      the formatted description string
     */
    private String formatDescription( String sXML )
    {
        StringBuffer sb = new StringBuffer();
        StringTokenizer st = new StringTokenizer( sXML.trim(), "\n" );
        int iNewLineStart = -1;
        while ( st.hasMoreTokens() )
        {
            String sNextLine = st.nextToken().trim();

            //replace series of empty spaces (created by the XMLSerializer)
            //with a single empty space
            StringTokenizer st2 = new StringTokenizer( sNextLine, " " );
            StringBuffer sb2 = new StringBuffer();
            while ( st2.hasMoreTokens() )
                sb2.append( st2.nextToken() ).append( " " );
            sNextLine = sb2.toString();

            //replace "lonely backslashes" within the text with newlines
            if ( ( iNewLineStart = sNextLine.indexOf( '\\' ) ) >= 0 )
            {
                sb.append( sNextLine.substring( 0, iNewLineStart ).trim() );
                sb.append( "\n" );
                sb.append( sNextLine.substring( iNewLineStart + 1 ).trim() );
            }
            else
                sb.append( sNextLine ).append( " " );
        }
        return sb.toString();
    }

    /**
     * Returns the jdbc element within the config file.
     *
     * @return   the class representing the jdbc element/tag
     */
    public JDBC getJDBC()
    {
        return jdbc;
    }

    /**
     * Returns the directory element within the config file.
     *
     * @return   the class representing the directory element/tag
     */
    public Directory getDirectory()
    {
        return directory;
    }

    /**
     * Returns the index element within the config file.
     *
     * @return   the class representing the index element/tag
     */
    public Index getIndex()
    {
        return index;
    }

    /**
     * Returns the system element within the config file.
     *
     * @return   the class representing the system element/tag
     */
    public org.alltimeflashdreamer.filelister.xml.System getSystem()
    {
        return system;
    }

    /**
     * Returns the link element within the config file.
     *
     * @return   the class representing the link element/tag
     */
    public Link getLink()
    {
        return link;
    }

    /**
     * Returns the zip element within the config file.
     *
     * @return   the class representing the zip element/tag
     */
    public Zip getZip()
    {
        return zip;
    }
}
